(* ::Package:: *)

(* ::Section:: *)
(*DebugTrace package*)


$DebugTrace=True;


BeginPackage["DebugTrace`"];


DTDebugExecute::usage="DTDebugExecute[command] \[Dash] executes a command under control of the debugger. If the command executes code already loaded by DTRunDebugger, the debugger will be entered.";


DTRunDebugger::usage="DTRunDebugger[file] \[Dash] Runs the notebook (.nb) or package (.m) file through the debugger.";


DTLocalValue::usage="DTLocalValue[n] \[Dash] Returns the value of the most recently created local variable of the form n$<number>";


DTLocalSymbols::usage="";


DTDynamicResults::usage="DTDynamicResults[opts] \[Dash] Returns list of debug output from dynamic computations. Use the option \"Keep\"->True to preserve this list";


DTPrint::usage="DTPrint[x]\[Dash] prints x=<result of evaluating x> and returns the evaluation result without evaluating it a second time (can be used inside or outside of debug sessions)";


DebugTraceHelp::usage="DebugTraceHelp[] \[Dash] Displays the help notebook";


DebugTraceVersion[]="1.51";


DTDebuggerInterrupt::usage="DTDebuggerInterrupt[mes] \[Dash] Forces the debugger to pause with the given message \[Dash] useful to stop at an interesting point with a minimum of fuss";


Begin["`Private`"]


(* ::Section:: *)
(*Initialisation*)


main\[Breve]palette\[Breve]title="DebugTrace control panel";
source\[Breve]notebook\[Breve]title="DebugTrace: Source code";
conditionals\[Breve]notebook\[Breve]title="Add/Remove conditional breakpoints";

UNAVAILABLE\[Breve]BP=0;
INACTIVE\[Breve]BP=1;
ACTIVE\[Breve]BP=2;
INACTIVE\[Breve]SELECTED\[Breve]BP=3;
ACTIVE\[Breve]SELECTED\[Breve]BP=4;

notebook\[Breve]stagger=100;

SetAttributes[sym,HoldAll];
SetAttributes[sym1,HoldAll];

Debug\[Breve]Print=SuperWidgetPackage`Debug\[Breve]Print;
Debug\[Breve]Box=SuperWidgetPackage`Debug\[Breve]Box;

multi\[Breve]step=100;

source\[Breve]nb=Null;

SWP\[Breve]Modal=False;

button\[Breve]colours={Gray,Blue,Red,Darker[Green],Orange};

message\[Breve]heads=Null;

dynamic\[Breve]debug\[Breve]count=0;

debugging\[Breve]selection=False;

dynamic\[Breve]results={};

replacement\[Breve]message=Null;

recent\[Breve]variables={};

recent\[Breve]functions={};

processing\[Breve]message=False;

standard\[Breve]header=Cell[BoxData[RowBox[{StyleBox["DebugTrace ",FontSize->18],
            StyleBox["Ver "<> DebugTraceVersion[],FontSize->10]}]], "Text",
      Background->RGBColor[0,1,1],ShowCellBracket->False,
      CellMargins->{{0,0},{0,0}}];

If[$VersionNumber<7,
   remove\[Breve]duplicates=Union,
      remove\[Breve]duplicates=DeleteDuplicates
   ];


str\[Breve]path=Null;
DebugTracePath[]:=Module[{ans},
   If[str\[Breve]path===Null,
      (* Do this carefully \[Dash] deep searches are expensive! *)
      str\[Breve]path=Module[{} ,
         ans=ToFileName[{$BaseDirectory,"Applications"},"DebugTrace"];
         If[FileType[ans]===Directory,Return[ans]];
         ans=ToFileName[{$UserBaseDirectory,"Applications"},"DebugTrace"];
         If[FileType[ans]===Directory,Return[ans]];
         ans=Select[FileNames["DebugTrace.m",$Path,4],
            StringTake[#,1]!="."&];
         If[ans==={},
            ans=Select[FileNames["DebugTrace.m",$Path,5],
               StringTake[#,1]!="."&]];
         StringDrop[ans[[1]],-21]
         ]
      ];
   str\[Breve]path
   ];


palette\[Breve]path=DebugTracePath[]<>$PathnameSeparator<>"FrontEnd"<>$PathnameSeparator<>
"Palettes"<>$PathnameSeparator<>"DebugTraceControlPanel.nb"


Window\[Breve]Title[nb_]:=("WindowTitle"/.NotebookInformation[nb]);


debugger\[Breve]help\[Breve]path=DebugTracePath[]<>$PathnameSeparator<>
"Documentation"<>$PathnameSeparator<>"English"<>$PathnameSeparator<>"Guides"<>$PathnameSeparator<>"DebugTrace.nb";


(* ::Section:: *)
(*Temporary fix for 9.0 bug*)


NotebookRead1[nb_]:=(NotebookRead[nb]/.RowBox[x___]/;(Length[{x}]!=1 || Head[x]=!=List):>RowBox[{x}]);
NotebookGet1[nb_]:=(NotebookGet[nb]/.RowBox[x___]/;(Length[{x}]!=1 || Head[x]=!=List):>RowBox[{x}]);



(*nbg\[Breve]nested=False;
Unprotect[NotebookGet];
NotebookGet[nb_]:=Block[{nbg\[Breve]nested->True},
NotebookGet[nb]/.RowBox[x___]/;(Length[{x}]!=1 || Head[x]=!=List):>RowBox[{x}];
]/;(!nbg\[Breve]nested);
Protect[NotebookGet];*)


(*Unprotect[NotebookRead];
NotebookRead[nb_]:=Block[{nbg\[Breve]nested->True},
NotebookRead[nb]/.RowBox[x___]/;(Length[{x}]!=1 || Head[x]=!=List):>RowBox[{x}];
]/;(!nbg\[Breve]nested);
Protect[NotebookRead];*)


(* ::Section:: *)
(*Preserved settings*)


settings\[Breve]file=$UserBaseDirectory<>$PathnameSeparator<>"DebugTrace.ini";

read\[Breve]settings\[Breve]file[]=Module[{data},
   data=Quiet[Get[settings\[Breve]file]];
   If[Head[data]===List,
      recent\[Breve]variables="RecentVariables"/.data/."RecentVariables"->{};
      recent\[Breve]functions="RecentFunctions"/.data/."RecentFunctions"->{};
      ];
   ];

write\[Breve]settings\[Breve]file[]:=Module[{},
   Quiet[DeleteFile[settings\[Breve]file]];
   Block[{$ContextPath={},$Context="Garbage`"},
      Put[{"RecentVariables"->recent\[Breve]variables,"RecentFunctions"->recent\[Breve]functions},settings\[Breve]file];
      ];
   ];

read\[Breve]settings\[Breve]file[];


(* ::Section:: *)
(*Compound GUI controls*)


unhold[var_]:=StringDrop[StringDrop[ToString[var,OutputForm],5],-1];


SetAttributes[edit\[Breve]list\[Breve]control,HoldFirst];
edit\[Breve]list\[Breve]control[var_Symbol,ll_List]:=Module[{btns},
   If[Length[ll]>0,var=ll[[1]];expr=unhold[var],var=Hold[Null];expr=""];
   btns=MapIndexed[Button[unhold[#],var=ll[[#2[[1]]]];expr=unhold[var],Appearance->"Palette",Alignment->Left]&,ll];
   Column[{
         Initial\[Breve]InputField[Dynamic[expr,(expr=#1;var=ToExpression[expr,StandardForm,Hold])&],String],
            Sequence@@btns
         },Spacings->{0,0},Frame->True]
   ];


Initial\[Breve]InputField[x___]:=InputField[x,If[$VersionNumber>=7,
      BoxID->"Start",Sequence@@{}]];


Options[Goto\[Breve]Start\[Breve]Box]={"Selected"->False};
Goto\[Breve]Start\[Breve]Box[nb_,OptionsPattern[Goto\[Breve]Start\[Breve]Box]]:=If[$VersionNumber>=7,
   MathLink`CallFrontEnd[FrontEnd`BoxReferenceFind[FE`BoxReference[nb,
            {{"Start"}},FE`BoxOffset->{FE`BoxChild[1]}],FE`SearchStart->"StartFromBeginning"]];
   If[! OptionValue["Selected"],FrontEndTokenExecute[nb,"Start"]];
   ];


(* ::Section:: *)
(*Palette*)


(* ::Text:: *)
(*Creates the palette \[Dash] for development use only*)


Create\[Breve]Palette[]:=Module[{tmp,padding},
   tmp=Select[Notebooks[],
      (Window\[Breve]Title[#]==main\[Breve]palette\[Breve]title)&];
   If[tmp =!={},
      NotebookClose[tmp[[1]]];
      ];
   padding=Button["",Null,Enabled->False];
   If[! ValueQ[step\[Breve]counter],step\[Breve]counter=0];
   main\[Breve]palette=CreatePalette[{standard\[Breve]header,
         Grid[Partition[
               {Tooltip[Button[Style["DebugTrace Help",Red],                     
                        Needs["DebugTrace`"];DebugTrace`DebugTraceHelp[],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Open help information"],

                  SpanFromLeft,

                  Tooltip[Button[Style["Debug this notebook",Red],                     
                        Needs["DebugTrace`"];DTRunDebugger[Evaluate[SelectedNotebook[]]],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Debug selected notebook"],

                  Tooltip[Button[Style["Debug selection",Red],                     
                        Needs["DebugTrace`"];debug\[Breve]selection[],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Debug selected cell(s)"],

                  Tooltip[Button["Debug package",
                        Needs["DebugTrace`"];DTRunDebugger[Evaluate[SelectedNotebook[]],"Package"->True],
                           Enabled->True,
                           Evaluator->Automatic,
                           Method->"Queued"],"Debug package or other called .m code"],

                  Tooltip[Button["Kill kernel",                     
                        If[ChoiceDialog["Also remove DebugTrace generated notebooks?"],Close\[Breve]Generated\[Breve]Notebooks[]];Quit[],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Kill the kernel for a clean restart,\noptionally removing all notebooks generated by DebugTrace"],

                  Tooltip[Button["Run until break",
                        If[in\[Breve]dialog,in\[Breve]dialog=False;stepping=False;ExitDialog[]],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Continue program to completion, or until breakpoint"],

                  Tooltip[Button["Step once",
                        If[in\[Breve]dialog,in\[Breve]dialog=False;ExitDialog[]],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run program to next step"],

                  Tooltip[Button[Dynamic[Style["Message interception", If[message\[Breve]interception\[Breve]required,Red,Black,Black]]],  
                        message\[Breve]interception\[Breve]required=(message\[Breve]interception\[Breve]required=!=True);
                        message\[Breve]interception=message\[Breve]interception\[Breve]required,
                           Enabled->Dynamic[in\[Breve]dialog,
                              Evaluator->Automatic],
                           Method->"Queued"],
                        "Toggle breakpoints on messages"],

                  Tooltip[Button["Run to step",
                        step\[Breve]at=Input["Enter step number:",0];
                        stepping=False;
                        ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run to specific step number\n(typically determined on a previous debug run)"],

                  Tooltip[Button["Print N steps",                        
                        step\[Breve]at=step\[Breve]counter+multi\[Breve]step;
                        stepping=False;
                        print\[Breve]steps\[Breve]regardless=True;
                        ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Print the next N steps but don't pause for interaction"],

                  Tooltip[Button["Run to function",
                        If[setup\[Breve]halt\[Breve]function[],
                           stepping=False;
                           ExitDialog[]
                           ],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run to next call to named function"],

                  Tooltip[Button["Run to variable",
                        If[setup\[Breve]halt\[Breve]variable[],
                           stepping=False;
                           ExitDialog[]
                           ],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run to next use or change of named variable"],

                  Tooltip[Button["Skip N steps",
                        step\[Breve]at=step\[Breve]counter+multi\[Breve]step;
                        stepping=False;
                        ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run for N steps (default 100 - see Set N button)\n without interaction"],

                  Tooltip[Button["Interrupt",
                        stepping=True,
                           Enabled->Dynamic[! in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic],
                        "Interupt a running program\n(e.g. after pressing Continue)"],

                  Tooltip[Button["Quit",
                        task=task\[Breve]abort;ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Stop the program completely"],

                  Tooltip[Button["Trace",
                        task=task\[Breve]trace;ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Print current stack of function call information"],

                  Tooltip[Button["Set N",
                        multi\[Breve]step=Input["Enter N for block steps:",multi\[Breve]step];,
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Adjust N, the jump size"],

                  Tooltip[Button[Dynamic[Style["Record N dynamics",If[dynamic\[Breve]debug\[Breve]count>0,Red,Black,Black]]],  
                        debug\[Breve]dynamics[],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Toggle continuous debugging for dynamics"],

                  Tooltip[Button["Step over",
                        If[in\[Breve]dialog,in\[Breve]dialog=False;task=task\[Breve]step\[Breve]over;ExitDialog[]],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Run program to next step"],

                  Tooltip[Button["Print dynamics",  
                        print\[Breve]dynamics[],
                           Enabled->Dynamic[dynamic\[Breve]results=!={}],
                           Evaluator->Automatic,
                           Method->"Queued"],

                     "Toggle continuous debugging for dynamics"],
                     Button[Dynamic[Style[If[SWP\[Breve]Modal===True,"SWP break","Private"],If[SWP\[Breve]Modal===True,Red,Gray]]],
                        If[SWP\[Breve]Modal,
                           stepping=True;
                        ToExpression["SuperWidgetPackage`Private`debugger\[Breve]interrupt=True"]],
                        Enabled->Dynamic[! in\[Breve]dialog],
                        Evaluator->Automatic],

                  Tooltip[Button["Conditionals",
                        task=task\[Breve]conditionals;ExitDialog[],
                           Enabled->Dynamic[in\[Breve]dialog,SynchronousUpdating->False,TrackedSymbols -> Full],
                           Evaluator->Automatic,
                           Method->"Queued"],
                        "Add/remove conditional breaakpoints"],

                  Tooltip[Button[Dynamic[Style["Execution timing", If[step\[Breve]timing,Red,Black,Black]]],  
                        step\[Breve]timing=(step\[Breve]timing=!=True),                           
                           Method->"Queued"],
                        "Toggle reporting of code timing information"],

                  (* Dummy entry to balance palette - comment out when not required *)
                  (*Button["",
                     Null,
                        Enabled->False,
                        Evaluator->Automatic,
                        Method->"Queued"], *)

                  Item[Style[Dynamic[If[!ValueQ[status\[Breve]info],status\[Breve]info="INACTIVE"];" Status: "<>status\[Breve]info,TrackedSymbols -> Full],Red],Alignment->Left],
                     Item[Dynamic[If[!ValueQ[stepping\[Breve]info],stepping\[Breve]info=""];stepping\[Breve]info,TrackedSymbols -> Full],Alignment->Left]},2]              
            ]},
         WindowFrameElements->{"CloseBox"},
         WindowElements->{(*"StatusArea",*)"MenuBar"},
         WindowTitle->main\[Breve]palette\[Breve]title];
   NotebookSave[main\[Breve]palette,palette\[Breve]path];
   ];



Close\[Breve]Generated\[Breve]Notebooks[]:=Module[{nn,ans,j},
   nn=Notebooks[];
   j=1;
   While[j <= Length[nn],
      ans=Window\[Breve]Title[nn[[j]]];
      If[StringQ[ans] && StringPosition[ans,"DebugTrace:"]=!={},
         NotebookClose[nn[[j]]]
         ];
      j++;
      ];
   ];


DebugTraceHelp[]:=NotebookOpen[debugger\[Breve]help\[Breve]path];


(* ::Section:: *)
(*Source/Breakpoint window*)


clear\[Breve]source[]:=Module[{},
   NotebookClose/@Select[Notebooks[],
      (Window\[Breve]Title[#]==source\[Breve]notebook\[Breve]title)&];
   ];


SetAttributes[brswap,HoldFirst];
brswap[var_]:=(var=Switch[var,
      UNAVAILABLE\[Breve]BP,UNAVAILABLE\[Breve]BP,
         INACTIVE\[Breve]BP,ACTIVE\[Breve]BP,
         ACTIVE\[Breve]BP,INACTIVE\[Breve]BP,
         INACTIVE\[Breve]SELECTED\[Breve]BP,ACTIVE\[Breve]SELECTED\[Breve]BP,
         ACTIVE\[Breve]SELECTED\[Breve]BP,INACTIVE\[Breve]SELECTED\[Breve]BP]);

SetAttributes[brselect,HoldFirst];
brselect[var_]:=(var=Switch[var,
      UNAVAILABLE\[Breve]BP,UNAVAILABLE\[Breve]BP,
         INACTIVE\[Breve]BP,INACTIVE\[Breve]SELECTED\[Breve]BP,
         ACTIVE\[Breve]BP,ACTIVE\[Breve]SELECTED\[Breve]BP,
         INACTIVE\[Breve]SELECTED\[Breve]BP,INACTIVE\[Breve]SELECTED\[Breve]BP,
         ACTIVE\[Breve]SELECTED\[Breve]BP,ACTIVE\[Breve]SELECTED\[Breve]BP]);

SetAttributes[bractivate,HoldFirst];
bractivate[var_]:=(var=Switch[var,
      UNAVAILABLE\[Breve]BP,UNAVAILABLE\[Breve]BP,
         INACTIVE\[Breve]BP,ACTIVE\[Breve]BP,
         ACTIVE\[Breve]BP,ACTIVE\[Breve]BP,
         INACTIVE\[Breve]SELECTED\[Breve]BP,ACTIVE\[Breve]SELECTED\[Breve]BP,
         ACTIVE\[Breve]SELECTED\[Breve]BP,ACTIVE\[Breve]SELECTED\[Breve]BP]);

SetAttributes[brdeselect,HoldFirst];
brdeselect[var_]:=(var=Switch[var,
      UNAVAILABLE\[Breve]BP,UNAVAILABLE\[Breve]BP,
         INACTIVE\[Breve]BP,INACTIVE\[Breve]BP,
         ACTIVE\[Breve]BP,ACTIVE\[Breve]BP,
         INACTIVE\[Breve]SELECTED\[Breve]BP,INACTIVE\[Breve]BP,
         ACTIVE\[Breve]SELECTED\[Breve]BP,ACTIVE\[Breve]BP]);


re\[Breve]colour\[Breve]line[ln_]:=Module[{lstr=line\[Breve]number\[Breve]string[ln]},
   If[last\[Breve]colour[ln]==breakpoint\[Breve]id[ln],Return[]];
   last\[Breve]colour[ln]=breakpoint\[Breve]id[ln];
   SetOptions[source\[Breve]nb,Editable->True];
   SelectionMove[source\[Breve]nb,Before,Notebook,AutoScroll->False];
   NotebookFind[source\[Breve]nb,lstr];Table[SelectionMove[source\[Breve]nb,All,Expression],{2}];
   NotebookWrite[source\[Breve]nb,test=(NotebookRead1[source\[Breve]nb]/.{"Output",Bold,_}->{"Output",Bold,button\[Breve]colours[[breakpoint\[Breve]id[ln]+1]]}),After];
   SetOptions[source\[Breve]nb,Editable->False];
   ];


break\[Breve]toggle[lno_]:=Which[
   CurrentValue["ControlKey"],
      If[in\[Breve]dialog,
         stepping=False;
      brswap@breakpoint\[Breve]id[lno];
      re\[Breve]colour\[Breve]line[lno];
      one\[Breve]shot\[Breve]line=lno;
      in\[Breve]dialog=False;
      ExitDialog[]
      ],

   CurrentValue["ShiftKey"],
      bractivate@breakpoint\[Breve]id[lno];
   re\[Breve]colour\[Breve]line[lno];
   adjust\[Breve]bp\[Breve]expressions[lno],

   True,
      brswap@breakpoint\[Breve]id[lno];re\[Breve]colour\[Breve]line[lno]
   ];


line\[Breve]number\[Breve]string[ln_]:=StringDrop[ToString[(ln)+10000],1]<>":";


dynamic\[Breve]buttons=False;

next\[Breve]line[]:=With[{lstr=line\[Breve]number\[Breve]string[lno]},RowBox[{
         StyleBox["(*",FontSize->0,FontColor->RGBColor[0,0,0,0]],
            With[{lno=lno++},TagBox[ToBoxes@                  
               Button[lstr,break\[Breve]toggle[lno],Appearance->None,
                  BaseStyle->{"Output",Bold,If[reachable[lno]===True,Blue,Gray]}],lno]
            ],
            StyleBox["*)",FontSize->0,FontColor->RGBColor[0,0,0,0]]
         }]
   ];


(* ::Section:: *)
(*Process box forms*)


indenting\[Breve]newline=ToExpression["\"\\[IndentingNewLine]\""];
embed\[Breve]line\[Breve]numbers["\n"]:=(current\[Breve]line++;"\n");
With[{indenting\[Breve]newline=indenting\[Breve]newline},embed\[Breve]line\[Breve]numbers[indenting\[Breve]newline]:=(current\[Breve]line++;"\n")];
embed\[Breve]line\[Breve]numbers[RowBox[ll_List]]:=Module[{top=current\[Breve]line,ans},
   ans=RowBox[Map[embed\[Breve]line\[Breve]numbers,ll]];
   If[!MemberQ[ans[[1]],","],ans=RowBox[{"DebugTrace`Private`Line\[Breve]Tag","[",ToString[top],",",ans,"]"}]];
   If[MemberQ[ll,"(*"],ans=RowBox[ll]];
   ans
   ]/; (Length[ll]>=1 && ll[[1]] =!= "[");

embed\[Breve]line\[Breve]numbers[x_]:=x;


process\[Breve]executable\[Breve]item[cc_RowBox]:=Module[{expr},
   current\[Breve]line=lno;
   If[! MatchQ[cc,RowBox[{"(*",___}]],reachable[lno]=True];
   expr=embed\[Breve]line\[Breve]numbers[cc];
   If[! MatchQ[expr,RowBox[{"(*",___}]],prog=Append[prog,expr]];
   With[{indenting\[Breve]newline=indenting\[Breve]newline}, RowBox[{next\[Breve]line[],cc}]/.{"\n":>Sequence["\n",next\[Breve]line[]],indenting\[Breve]newline:>Sequence["\n",next\[Breve]line[]]}]
   ];
process\[Breve]executable\[Breve]item[x_]:=x;


process\[Breve]cell[cc:Cell[BoxData[_RowBox],___]]:=(
   cc/.BoxData[x_RowBox]:>BoxData[process\[Breve]executable\[Breve]item[x]]
   );
process\[Breve]cell[cc:Cell[BoxData[_List],___]]:=(
   cc/.BoxData[x_List]:>BoxData[Map[process\[Breve]executable\[Breve]item,x]]
   );
process\[Breve]cell[cc:Cell[BoxData[_String],___]]:=cc;


(* ::Section:: *)
(*Annotate code*)


(*debug\[Breve]annotate[x_]:=0/;(Print[x];False); *)

debug\[Breve]annotate[sym[SetDelayed][Line\[Breve]Tag[ln_,head:sym[fn_Symbol][args___]],body_]]:=Module[{b1},
   If[condition\[Breve]bodyQ[body],
      b1=body;
      If[Head[b1]===Line\[Breve]Tag,b1=b1[[2]]];
      sym[SetDelayed][head,sym[Condition][sym[Function\[Breve]Tag][wrap\[Breve]args[debug\[Breve]annotate[b1[[1]]],{args}],SymbolName[fn],ln],b1[[2]]]],
         sym[SetDelayed][head,sym[Function\[Breve]Tag][wrap\[Breve]args[debug\[Breve]annotate[body],{args}],SymbolName[fn],ln]]
      ]
   ]/; fn=!=Part;

debug\[Breve]annotate[sym[SetDelayed][head:sym[var_Symbol],Line\[Breve]Tag[ln_,body_]]]:=
   sym[SetDelayed][head,sym[Delayed\[Breve]variable\[Breve]Tag][debug\[Breve]annotate[Line\[Breve]Tag[ln,body]],SymbolName[var],ln]];
(*debug\[Breve]annotate[sym[SetDelayed][head:sym[var_Symbol],body_]]:=
   sym[SetDelayed][head,sym[Delayed\[Breve]variable\[Breve]Tag][debug\[Breve]annotate[body],SymbolName[var],7]];*)

debug\[Breve]annotate[sym[UpSetDelayed][Line\[Breve]Tag[ln_,head:sym[fn_Symbol][args___]],body_]]:=
   sym[UpSetDelayed][head,sym[Function\[Breve]Tag][wrap\[Breve]args[debug\[Breve]annotate[body],{args}],SymbolName[fn],ln]];

debug\[Breve]annotate[sym[TagSetDelayed][tag_,Line\[Breve]Tag[ln_,head:sym[fn_Symbol][args___]],body_]]:=
   sym[TagSetDelayed][tag,head,sym[Function\[Breve]Tag][wrap\[Breve]args[debug\[Breve]annotate[body],{args}],SymbolName[fn],ln]];

debug\[Breve]annotate[sym[Set][head_,Line\[Breve]Tag[_,body_]]]:=
   sym[Set][head,debug\[Breve]annotate[body]];

debug\[Breve]annotate[sym[UpSet][head_,Line\[Breve]Tag[_,body_]]]:=
   sym[UpSet][head,debug\[Breve]annotate[body]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[Module][mlist_,mbody_]]]:=sym[Module][mlist,
   sym[CompoundExpression][
      sym[Debug\[Breve]Tag1][ln,"In Module with local variables ","ExtraInfo"->sym[HoldForm][mlist/.sym[Set][a_,b_]->a]],
         debug\[Breve]annotate[mbody]
      ]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[DynamicModule][mlist_,mbody_]]]:=sym[DynamicModule][mlist,
   sym[CompoundExpression][
      sym[Debug\[Breve]Tag1][ln,"In DynamicModule with local variables ","ExtraInfo"->sym[HoldForm][mlist/.sym[Set][a_,b_]->a]],
         debug\[Breve]annotate[mbody]
      ]];

debug\[Breve]annotate[sym[Block][mlist_,mbody_]]:=sym[Block][mlist,debug\[Breve]annotate[mbody]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[For][start_,test_,incr_,body_]]]:=
   sym[For][start,test,incr,sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"Executing body of For"],debug\[Breve]annotate[body]]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[Do][body_,iter_]]]:=
   sym[Do][sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"Executing body of Do"],debug\[Breve]annotate[body]],iter];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[Do][x___]]]:=sym[Debug\[Breve]Tag1][ln,"Syntactically invalid Do construct"];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[While][test_,body_]]]:=
   sym[While][test,sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"Executing body of While"],debug\[Breve]annotate[body]]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[Switch][val_,cases__]]]:=
   sym[Switch][val,Sequence@@
   MapIndexed[If[EvenQ[#2[[1]]],debug\[Breve]annotate[#1],#1]&,{cases}]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[Which][cases__]]]:=
   sym[Which][Sequence@@
   (*MapIndexed[If[EvenQ[#2[[1]]],debug\[Breve]annotate[#1],#1]&,{cases}]];*)
   MapIndexed[debug\[Breve]annotate[#1]&,{cases}]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[If][test_,body_]]]:=
   sym[If][test,sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"If test True"],debug\[Breve]annotate[body]]];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,sym[If][test_,body1_,body2_]]]:=
   sym[If][test,sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"If test True"],debug\[Breve]annotate[body1]],
      sym[CompoundExpression][sym[Debug\[Breve]Tag1][ln,"If test False"],debug\[Breve]annotate[body2]]];

(*debug\[Breve]annotate[sym[List][args__]]:=sym[List]@@Map[debug\[Breve]annotate,{args}];*)

debug\[Breve]annotate[sym[h_Symbol][args__]]:=sym[h]@@MapIndexed[If[processable[sym[h],#2[[1]]],debug\[Breve]annotate1[#,If[Context[h]=="System`",0,#2[[1]]]],#]&,{args}];

debug\[Breve]annotate[sym[CompoundExpression][exprs__]]:=sym[CompoundExpression]@@Map[debug\[Breve]annotate,{exprs}];

debug\[Breve]annotate[Line\[Breve]Tag[ln_,expr_]]:=sym[CompoundExpression][sym[Debug\[Breve]Tag][ln,""],debug\[Breve]annotate[expr]];

debug\[Breve]annotate[expr_]:=expr;

debug\[Breve]annotate1[Line\[Breve]Tag[_,expr_],1]:=debug\[Breve]annotate[expr];
debug\[Breve]annotate1[expr_,_]:=debug\[Breve]annotate[expr];



define\[Breve]break\[Breve]location[ln_]:=Mosule[{},
   (* If[ToExpression["DebugTrace`Private`bp"<>ToString[ln]]>0,Return[]];
   ToExpression["DebugTrace`Private`bp"<>ToString[ln]<>"=DebugTrace`Private`INACTIVE\[Breve]BP"];*)
   If[breakpoint\[Breve]id[ln]>0,Return[]];
   breakpoint\[Breve]id[ln]=INACTIVE\[Breve]BP;
   re\[Breve]colour\[Breve]line[ln];
   ];


processable[sym[x_],n_]:=Module[{aa=Attributes[x]},
   If[MemberQ[aa,HoldAll],Return[False]];
   If[MemberQ[aa,HoldAllComplete],Return[False]];
   If[n==1 && MemberQ[aa,HoldFirst],Return[False]];
   If[n>1 && MemberQ[aa,HoldRest],Return[False]];
   True
   ];


SetAttributes[full\[Breve]symbol\[Breve]name,HoldFirst];
full\[Breve]symbol\[Breve]name[x_]:=Context[x]<>SymbolName[Unevaluated[x]];


condition\[Breve]bodyQ[sym[Condition][___]]:=True;
condition\[Breve]bodyQ[Line\[Breve]Tag[_,sym[Condition][___]]]:=True;
condition\[Breve]bodyQ[_]:=False;


wa[sym[x_]]:=Module[{tmp},
   tmp=sym[Evaluate[ToExpression[full\[Breve]symbol\[Breve]name[x]<>"$DebugTrace$"]]];
   sym[Set][tmp,sym[x]]
   ];

wa1[sym[x_]]:=Module[{tmp},
   tmp=sym[Evaluate[ToExpression[full\[Breve]symbol\[Breve]name[x]<>"$DebugTrace$"]]];
   sym[Set][tmp,AngleBracket[sym[x]]]
   ];

wrap\[Breve]args[body_,args_]:=Module[{ss,ans},
   ss=
      ss=Join[Map[wa,Cases[args,sym[Pattern][x_,sym[Blank][___]]->x,\[Infinity]]],
         Map[wa1, Cases[args,sym[Pattern][x_,sym[BlankSequence][___]]->x,\[Infinity]]],
         Map[wa1,Cases[args,sym[Pattern][x_,sym[BlankNullSequence][___]]->x,\[Infinity]]]
      ];
   If[ss==={},Return[body]];
   ans=sym1[Module][ss,body];
   ans
   ];


(* ::Section::Closed:: *)
(*Clean up multiple references to the same line*)


clean\[Breve]up[sym[Debug\[Breve]Tag][ln_,description_,opts___]]:=(
   (*If[cleaned\[Breve]up[ln],Return[clean\[Breve]up[expr]]];*)
   If[cleaned\[Breve]up[ln],Return[empty]];
   cleaned\[Breve]up[ln]=True;
   sym[Debug\[Breve]Tag][ln,description,opts]
   );
clean\[Breve]up[sym[fn_][body___]]:=sym[fn][Sequence@@Map[clean\[Breve]up,{body}]]/;(fn =!=Debug\[Breve]Tag);
clean\[Breve]up[sym1[fn_][body___]]:=sym1[fn][Sequence@@Map[clean\[Breve]up,{body}]];
clean\[Breve]up[Line\[Breve]Tag[ln_,x_]]:=Line\[Breve]Tag[ln,clean\[Breve]up[x]];
clean\[Breve]up[expr_]:=expr;


(* ::Section:: *)
(*DTRunDebugger*)


debug\[Breve]selection[]:=Module[{nb=SelectedNotebook[],sel},
   sel=NotebookRead1[nb];
   If[Head[sel]=!=Cell && ! MatchQ[sel,{_Cell,___}],
      MessageDialog["You must select one or more cells for this operation",WindowTitle->"Error"];
      Return[];
      ];  
   debugging\[Breve]selection=True;
   DTRunDebugger[nb];
   ];


(*SyntaxInformation[DTRunDebugger]={"ArgumentsPattern"->{_String | _NotebookObject}};*)
Options[DTRunDebugger]={"Package"->False};
DTRunDebugger[file_String,OptionsPattern[DTRunDebugger]]:=Module[{},
   notes\[Breve]nb=SelectedNotebook[];
   If[FileType[file]===None,
      Print[Style[StringForm["***** Unable to open ``",file],Purple]];
      Abort[];
      ];
   
   program\[Breve]nb=NotebookOpen[file];
   If[$VersionNumber<7,
      program\[Breve]filename=file,
         program\[Breve]filename=ExpandFileName[file]
      ];
   (* NotebookOpen[palette\[Breve]path];*)
   display\[Breve]wait\[Breve]screen[];
   do\[Breve]debug[]
   ];


If[$VersionNumber<7,
   Unprotect[NotebookFileName];
   NotebookFileName[nb_NotebookObject]:=ToFileName@("FileName"/.NotebookInformation[nb]/.{"FileName"->""});
   ];


DTRunDebugger[nb_NotebookObject,OptionsPattern[DTRunDebugger]]:=Module[{packageQ,nn,j,ans},
   packageQ=OptionValue["Package"];
   program\[Breve]filename=Quiet[NotebookFileName[nb]];
   If[program\[Breve]filename===$Failed,program\[Breve]filename=StringDrop[StringDrop[ToString[nb],17],-3]]; 
   If[!packageQ,display\[Breve]wait\[Breve]screen[]];  
   notes\[Breve]nb=NotebookCreate[WindowTitle->"DebugTrace: Results"];
   SetOptions[notes\[Breve]nb,DockedCells->{standard\[Breve]header}];
   NotebookWrite[notes\[Breve]nb,
      {Cell["Debugging "<>If[debugging\[Breve]selection,"part of ",""]<>program\[Breve]filename,"Text"]}]; 
   program\[Breve]nb=nb; 
(*\[LightBulb]2*)
   If[packageQ,
      package\[Breve]name=StringReplace[StringDrop[FileNameTake[program\[Breve]filename,{-2,-1}],-2]<>"`",$PathnameSeparator->"`"];
(*Debug\[Breve]Print[package\[Breve]name,Background->RGBColor[0.6,0.6,1]];*)
      nn=Complement[Notebooks[],{main\[Breve]palette,MessagesNotebook[],program\[Breve]nb}];      
      j=1;
      ans=ChoiceDialog[Column[{
               "Select the test notebook to run package "<>package\[Breve]name,
                  PopupView[Map[Window\[Breve]Title,nn],Dynamic[j]]
               }]
         ];
      If[! ans,Return[]];
      testing\[Breve]nb=nn[[j]];
      System`Private`$InputFileName=NotebookFileName[testing\[Breve]nb];
      Unprotect[Get];
      
      With[{program\[Breve]filename=program\[Breve]filename},Get[x_/;((StringMatchQ[x,program\[Breve]filename,IgnoreCase->True]||StringMatchQ[x,package\[Breve]name,IgnoreCase->True]))]:=do\[Breve]debug["Package"->True]];
      ];
   With[{command=If[packageQ,"DebugTrace`Private`do\[Breve]delayed\[Breve]debug","DebugTrace`Private`do\[Breve]debug"]},
      NotebookWrite[notes\[Breve]nb,Cell[Boxdata[RowBox[{command,"[","]" }]],
            "Input",FontSize->0,ShowCellBracket->False,ShowSyntaxStyles->False,
               FontColor->RGBColor[0,0,0,0]]]
      ];
   SelectionMove[notes\[Breve]nb,Previous,Cell];
   SelectionEvaluate[notes\[Breve]nb];
   (* do\[Breve]debug[]*)
   ];


display\[Breve]wait\[Breve]screen[]:=(wait\[Breve]screen=CreateDocument[{standard\[Breve]header,
         TextCell["Please wait while the"],
            TextCell["debugger initialises"]
         },
         WindowFloating->True,WindowSize->All,
         Editable->False,
         WindowElements->{},ShowCellBracket->False,FontSize->16,WindowFrame->"Palette"]);


line\[Breve]cache={};
mark\[Breve]breakpoint\[Breve]locations[active\[Breve]lines_]:=(line\[Breve]cache=Join[line\[Breve]cache,active\[Breve]lines]);

update\[Breve]breakpoint\[Breve]locations[]:=Module[{},
   If[Length[line\[Breve]cache]>10,
      info=NotebookGet1[source\[Breve]nb];
      Map[(breakpoint\[Breve]id[#]=INACTIVE\[Breve]BP)&,line\[Breve]cache];
      info=info/.TagBox[bb_ButtonBox,ln_Integer]:>TagBox[bb/.{"Output",Bold,_}->{"Output",Bold,button\[Breve]colours[[breakpoint\[Breve]id[ln]+1]]},ln];
      NotebookPut[info,source\[Breve]nb],
         Map[define\[Breve]break\[Breve]location,line\[Breve]cache]
      ];
   line\[Breve]cache={};
   ];


(* ::Section:: *)
(*Delayed debugging for package testing*)


SetAttributes[en1,Listable];
en1[x_]:=Module[{tmp},tmp=ToExpression[x];If[tmp=!=Null,Print[tmp]]];

Execute\[Breve]Notebook[nb_]:=Module[{sss,tmp},
   sss=NotebookGet1[nb];
   sss=sss/.Cell[_,"Output",___]:>Sequence[];      
   sss=sss/.Cell[BoxData[x___],"Code" | "Input",___]:>Map[en1,{x}];
   ];


do\[Breve]delayed\[Breve]debug[]:=Module[{},
   Execute\[Breve]Notebook[testing\[Breve]nb]
   ]


(* ::Section:: *)
(*Top level execution loop*)


Options[do\[Breve]debug]={"Package"->False};
do\[Breve]debug[OptionsPattern[do\[Breve]debug]]:=Module[{sss,k,expr,active\[Breve]lines,ans,timeout\[Breve]option,nb,ds,tmp,packageQ},
   packageQ=OptionValue["Package"];
   If[packageQ,display\[Breve]wait\[Breve]screen[]]; 
   SetOptions[notes\[Breve]nb,WindowMargins->{{2*notebook\[Breve]stagger,Automatic},{Automatic,2*notebook\[Breve]stagger}}];  
   ds=debugging\[Breve]selection;   
   debugging\[Breve]selection=False; 
   Reset\[Breve]Debugger[];
   (* Reload of palette to avoid glitches after kernel kill *)
   nb=NotebookOpen[palette\[Breve]path,Visible->False];
   moption=Options[nb,WindowMargins];
   NotebookClose[nb];
   NotebookOpen[palette\[Breve]path,Sequence@@moption];
   status\[Breve]info="LOADING";
   clear\[Breve]source[];
   (* create\[Breve]palette[];*)
   clear\[Breve]conditionals[];
   Clear[breakpoint\[Breve]id];
   Clear[last\[Breve]colour];

   stepping=True;
   stepping\[Breve]over=False;
   step\[Breve]counter=0;
   step\[Breve]at=\[Infinity];
   
   stack\[Breve]base=Null;
   debugging=True;
   Catch[
      lno=1;
      prog={};
      Clear[reachable];
      sss=NotebookGet1[program\[Breve]nb];
      If[ds,
         tmp=NotebookRead1[program\[Breve]nb];
         If[Head[tmp]=!=List,tmp={tmp}];
         sss[[1]]=tmp;
         ];      
      sss=sss/.Cell[_,"Output",___]:>Sequence[];
      
      sss=sss/.cc:Cell[BoxData[___],"Code" | "Input",___]:>process\[Breve]cell[cc];
      Table[(breakpoint\[Breve]id[k]=If[reachable[k],INACTIVE\[Breve]BP,UNAVAILABLE\[Breve]BP,UNAVAILABLE\[Breve]BP]),{k,1,lno}];
      timeout\[Breve]option=First[Options[$FrontEnd,DynamicEvaluationTimeout]];
      SetOptions[$FrontEnd,DynamicEvaluationTimeout->1000];
      source\[Breve]nb=NotebookPut[sss,Editable->False,WindowTitle->source\[Breve]notebook\[Breve]title,Visible->False,Saveable->False];
      
      SetOptions[source\[Breve]nb,{WindowMargins->{{notebook\[Breve]stagger,Automatic},{Automatic,notebook\[Breve]stagger}},ClosingSaveDialog->False}];
      SetOptions[source\[Breve]nb,DockedCells->{standard\[Breve]header}];
      SetOptions[source\[Breve]nb,Visible->True];
      SetOptions[$FrontEnd,timeout\[Breve]option];
      k=1;
      NotebookClose[wait\[Breve]screen];
      debugger\[Breve]loaded=True;
      While[k <= Length[prog],
         expr=Quiet[Check[ToExpression[prog[[k]],StandardForm,Hold],syntax\[Breve]abort[prog[[k]]]]];
         expr=expr/.(s_Symbol/;Unevaluated[s]=!=Line\[Breve]Tag)->sym[s];
         expr[[1]]=debug\[Breve]annotate[expr[[1]]];
         Clear[cleaned\[Breve]up];
         expr=clean\[Breve]up[expr]//.Debug\[Breve]Tag1->Debug\[Breve]Tag;
         active\[Breve]lines=Union@Cases[expr,sym[Debug\[Breve]Tag][ln_,___]->ln,\[Infinity]];
         mark\[Breve]breakpoint\[Breve]locations[active\[Breve]lines];
         (* Various ad-hoc fixes *)
         expr=ad\[Breve]hoc\[Breve]fixes[expr];  
         expr=expr/.{sym[s_]->s,sym1[s_]->s} //.DebugTrace`Private`Line\[Breve]Tag[_,x_]:>x;               
         message\[Breve]interception=message\[Breve]interception\[Breve]required;
         ans=ReleaseHold[expr];
         If[ans=!=Null,Print[ans]];
         message\[Breve]interception=False;
         k++;
         ];
      update\[Breve]breakpoint\[Breve]locations[];
      If[packageQ,Return[]];
      Print[Style["****Code execution completed****",Purple]],
         debug\[Breve]abort\[Breve]tag
      ];
   Reset\[Breve]Debugger[];
   ];

clear\[Breve]conditionals[]:=Module[{},
   main\[Breve]palette=Null;
   NotebookClose/@Select[Notebooks[],
      (Window\[Breve]Title[#]==conditionals\[Breve]notebook\[Breve]title)&];
   ];

ad\[Breve]hoc\[Breve]fixes[expr_]:=Module[{},
   With[{program\[Breve]filename=program\[Breve]filename},
      expr /. sym[System`Private`$InputFileName]->
      sym[If][sym[System`Private`$InputFileName]=="",program\[Breve]filename,
         sym[System`Private`$InputFileName]]]//.empty:>Sequence@@{}
   //. sym[CompoundExpression][x_]->x
   ];




syntax\[Breve]abort[s_]:=Module[{lns},
   Print[s];
   lns=Map[ToExpression,Cases[s,RowBox[{"DebugTrace`Private`Line\[Breve]Tag",_,n_,___}]->n,\[Infinity]]];
   If[lns==={},lns=last\[Breve]line\[Breve]encountered,lns=Min[lns]];
   Print[Style["**** Syntax error on line "<>ToString[lns]<>" - DebugTrace aborted ****",Purple]];
   Throw["",debug\[Breve]abort\[Breve]tag];
   ];


(* ::Section::Closed:: *)
(*Run to function/variable*)


setup\[Breve]halt\[Breve]function[]:=Module[{fn,item,complete},
   fn=Hold[Null];
   complete=DialogInput[{standard\[Breve]header,
         TextCell["  Select function for break  ",
            FontSize -> 14, CellMargins -> {{0, 0}, {0, 0}}],
            edit\[Breve]list\[Breve]control[fn,recent\[Breve]functions],
            Row[{DefaultButton["OK", 
                  recent\[Breve]functions=remove\[Breve]duplicates[Prepend[recent\[Breve]functions,fn]];
                  If[Length[recent\[Breve]functions]>4,recent\[Breve]functions=Take[recent\[Breve]functions,4]];
                  write\[Breve]settings\[Breve]file[];
                  DialogReturn[True]],
                  DefaultButton["Cancel",DialogReturn[False]]}]}, WindowTitle ->"Select function for break",
         WindowSize -> All,
         WindowMargins -> {{Automatic, 0}, {Automatic, 500}},
         Saveable -> False,
         Evaluate[If[$VersionNumber>=7,Initialization:>(Goto\[Breve]Start\[Breve]Box[#,"Selected"->True]&),Sequence@@{}]],
         WindowElements -> {}, Modal->True];
   If[complete=!=True || fn===Hold[Null] || fn[[1,0]]=!=Symbol,Return[False]];
   fn=fn[[1]]; (* assumes function has no OwnValue *)
   item=With[{fn=fn},HoldPattern[fn[___]]:>Null/;
      (Function\[Breve]Break["Break on call to function ",HoldForm[fn]];False)];
   Unprotect[Evaluate[fn]];
   ll=DownValues[Evaluate[fn]];
   If[Length[ll]>0 && !FreeQ[First[ll],Debug\[Breve]Tag],ll=Drop[ll,1]];
   DownValues[Evaluate[fn]]=Prepend[ll,item];
   With[{fn=fn},data\[Breve]break\[Breve]active[HoldForm[fn]]=True];
   True
   ];


setup\[Breve]halt\[Breve]variable[]:=Module[{var,ll,complete},
   complete=DialogInput[{standard\[Breve]header,
         TextCell["  Select variable for break on next use or change ",
            FontSize -> 14, CellMargins -> {{0, 0}, {0, 0}}],
            edit\[Breve]list\[Breve]control[var,recent\[Breve]variables],
            Row[{DefaultButton["OK", 
                  recent\[Breve]variables=remove\[Breve]duplicates[Prepend[recent\[Breve]variables,var]];
                  If[Length[recent\[Breve]variables]>4,recent\[Breve]variables=Take[recent\[Breve]variables,4]];
                  write\[Breve]settings\[Breve]file[];DialogReturn[True]],
                  DefaultButton["Cancel",
                     DialogReturn[False]]}]}, WindowTitle ->"Select function for break",
         WindowSize -> All,
         WindowMargins -> {{Automatic, 0}, {Automatic, 500}},
         Evaluate[If[$VersionNumber>=7,Initialization:>(Goto\[Breve]Start\[Breve]Box[#,"Selected"->True]&),Sequence@@{}]],
         Saveable -> False,
         WindowElements -> {}, Modal->True];
   If[complete=!=True || var===Hold[Null](* || (var/.Hold[a_]->Head[Unevaluated[a]])=!=Symbol*),Return[False]];
   Apply[Unprotect,var];
   ll=Apply[OwnValues,var];
   If[FreeQ[ll,Debug\[Breve]Tag] && Length[ll]>0,
      ll[[1,2]]=With[{xx=ll[[1,2]]},sym[
            Data\[Breve]Break["Break on use of variable ",HoldForm@@var];
            xx
            ]];
      ll=ll/.sym[x_]->x;
      Apply[Clear,var];
      OwnValues[Evaluate[var[[1]]]]=ll;
      ]; 
   item=With[{hvar=sym@@var},sym1[Experimental`ValueFunction[hvar]=
         (Data\[Breve]Break["Break after change of value of variable ",HoldForm[hvar]]&)]];
   item/.sym[x_]->x/.sym1[x_]->x;
   With[{hvar=Apply[HoldForm,var]},data\[Breve]break\[Breve]active[hvar]=True];
   True
   ];


Data\[Breve]Break[mes_,var_]:=Module[{},
   If[data\[Breve]break\[Breve]active[var],
      data\[Breve]break\[Breve]active[var]=False;
      stepping=True;
      (*step\[Breve]counter--;*)
      replacement\[Breve]message=mes<>ToString[var]<>", which happened at line "<>ToString[last\[Breve]line\[Breve]encountered];
      (* Debug\[Breve]Tag[last\[Breve]line\[Breve]encountered,mes<>ToString[var]]*)
      ];
   ];


Function\[Breve]Break[mes_,var_]:=Module[{},
   If[data\[Breve]break\[Breve]active[var],
      data\[Breve]break\[Breve]active[var]=False;
      stepping=True;
      step\[Breve]counter--;
      (* replacement\[Breve]message=mes<>ToString[var]<>", which happened at line "<>ToString[last\[Breve]line\[Breve]encountered];*)
      Debug\[Breve]Tag[last\[Breve]line\[Breve]encountered,mes<>ToString[var]]
      ];
   ];


DTDebuggerInterrupt[mes_]:=Module[{},
   If[debugging && !in\[Breve]dialog,
      stepping=True;
      Debug\[Breve]Tag[last\[Breve]line\[Breve]encountered,"User interruption \[Dash] "<>mes];
      ];
   ];



(* ::Section:: *)
(*Run time interaction*)


Reset\[Breve]Debugger[]:=(
   set\[Breve]status\[Breve]line[];
   clear\[Breve]conditionals[];
   in\[Breve]dialog=False;
   one\[Breve]shot\[Breve]line=-1;
   step\[Breve]timing=False;
   time\[Breve]at\[Breve]step\[Breve]start=-1;
   status\[Breve]info="INACTIVE";
   last\[Breve]line\[Breve]encountered=0;
   in\[Breve]dialog=False;
   (*step\[Breve]counter=0;*)
   stepping=True;
   stepping\[Breve]over=False;
   task=Null;
   debugging=False;
   debugger\[Breve]loaded=False;
   message\[Breve]interception=False;
   message\[Breve]interception\[Breve]required=True;
   active\[Breve]breakpoints=True;
   all\[Breve]conditionals={};
   multi\[Breve]step=100;
   Clear[data\[Breve]break\[Breve]active];
   line\[Breve]cache={};
   );


main\[Breve]palette=Null;
Reset\[Breve]Debugger[];


message\[Breve]interception=False;
message\[Breve]interception\[Breve]required=True;
Unprotect[Message];
Message[mes_,extras___]:=Module[{},
   If[Head[mes]===$Off,Return[]];   
   latest\[Breve]diagnostic\[Breve]message=HoldForm[Message[mes,extras]];
   message\[Breve]interception=False;
   Message[mes,extras];
   message\[Breve]interception=True;
   If[Head[mes]===MessageName && mes[[2]]=="trace",Return[]];
   If[message\[Breve]heads =!= Null && ! MemberQ[message\[Breve]heads,Hold[mes][[1,1]]],Return[]]; 
   stepping=True;
   processing\[Breve]message=True;
   (* Don't treat the message interception as a step \[Dash] otherwise we get out of step *)
   step\[Breve]counter--;
   Debug\[Breve]Tag[last\[Breve]line\[Breve]encountered,"Break after message "<>ToString[HoldForm[mes]]];
   processing\[Breve]message=False;
   ]/;(message\[Breve]interception && ! in\[Breve]dialog);


info\[Breve]clean[0]:="outside the debugger";
info\[Breve]clean[x_]:=x;


Print\[Breve]Trace[]:=Module[{s=stack\[Breve]base},
   print\[Breve]db[Style[StringForm["In line ``",last\[Breve]line\[Breve]encountered],Darker[Green]]];
   While[s =!= Null,
      print\[Breve]db[Style[StringForm["In `` `` that starts at line `` , called from line ``",Sequence@@Map[info\[Breve]clean,Drop[s,1]]],Darker[Green]]];
      s=s[[1]];
      ]
   ]


print\[Breve]db[x___]:=Module[{},
   If[$DynamicEvaluation,
      dynamic\[Breve]results=Append[dynamic\[Breve]results,ToString[SequenceForm[x]]],
         Print[x]
      (*SelectionMove[notes\[Breve]nb,After,Notebook];
NotebookWrite[notes\[Breve]nb,Cell[BoxForm[ToBoxes@x],"Print"]];
SelectionMove[notes\[Breve]nb,After,Notebook];*)
      ];
   ];


Perform\[Breve]Abort[]:=Module[{},
   Print[Style["**** Quit by user ***",Purple]];
   Throw["",debug\[Breve]abort\[Breve]tag];
   ];      


bp\[Breve]breaks[_]:={};

adjust\[Breve]bp\[Breve]expressions[lno_]:=Module[{},
   bp\[Breve]expr="";
   NotebookClose[bp\[Breve]expression\[Breve]notebook];
   this\[Breve]bp\[Breve]breaklist=bp\[Breve]breaks[lno];
   bp\[Breve]expression\[Breve]notebook=CreateDocument[{standard\[Breve]header,
         TextCell["  Add or remove expressions for breakpoint at line "<>ToString[lno],
            FontSize -> 14, CellMargins -> {{0, 0}, {0, 0}}],
            Dynamic@Grid[Append[

               MapIndexed[{Button["Remove", drop\[Breve]bp\[Breve]expression[lno,#2[[1]]]], #} &,
                  this\[Breve]bp\[Breve]breaklist],
                  {Button["Add", add\[Breve]bp\[Breve]expr[lno,bp\[Breve]expr]],
                     InputField[Dynamic@bp\[Breve]expr, String]}], Alignment -> Left,
               Frame -> All],

         DefaultButton["Close", add\[Breve]bp\[Breve]expr[lno,bp\[Breve]expr];
            DialogReturn[]]}, WindowTitle -> "Breakpoint expressions",
         WindowSize -> All,
         WindowMargins -> {{Automatic, 0}, {Automatic, 500}},
         Saveable -> False,
         WindowElements -> {"StatusArea", "MemoryMonitor",
            "MagnificationPopUp", "MenuBar"}];
   SelectionMove[bp\[Breve]expression\[Breve]notebook, Next, Cell, 3];
   SelectionMove[bp\[Breve]expression\[Breve]notebook, Before, CellContents]; 
   SelectionMove[bp\[Breve]expression\[Breve]notebook, Next,Character, 12]; 
   ];

drop\[Breve]bp\[Breve]expression[lno_,n_]:=(
   bp\[Breve]expr="";
   bp\[Breve]breaks[lno]=Delete[bp\[Breve]breaks[lno],n];
   this\[Breve]bp\[Breve]breaklist=bp\[Breve]breaks[lno];
   );

add\[Breve]bp\[Breve]expr[lno_,ex_]:=Module[{ex1="Hold["<>ex<>"]",expr},
   If[StringMatchQ[ex,WhitespaceCharacter...],Return[]];
   If[! SyntaxQ[ex1],
      CreateDialog[{"Expression - invalid syntax",
            DefaultButton[]}];
      Return[];
      ];
   expr=ToExpression[ex1];
   bp\[Breve]breaks[lno]=Append[bp\[Breve]breaks[lno],expr/.Hold->HoldForm];
   this\[Breve]bp\[Breve]breaklist=bp\[Breve]breaks[lno];
   bp\[Breve]expr="";
   ];


Adjust\[Breve]Conditionals[]:=Module[{},
   cond\[Breve]expr="";
   clear\[Breve]conditionals[];
   conditionals\[Breve]notebook=CreateDocument[{standard\[Breve]header,
         TextCell["  Add or remove conditional breakpoints  ",
            FontSize -> 14, CellMargins -> {{0, 0}, {0, 0}}],
            Dynamic@Grid[Append[

               MapIndexed[{Button["Remove", drop\[Breve]conditional[#2[[1]]]], #} &,
                  all\[Breve]conditionals],
                  {Button["Add", add\[Breve]conditional[cond\[Breve]expr]],
                     Initial\[Breve]InputField[Dynamic@cond\[Breve]expr, String]}], Alignment -> Left,
               Frame -> All],

         DefaultButton["Close", add\[Breve]conditional[cond\[Breve]expr];
            DialogReturn[]]}, WindowTitle -> conditionals\[Breve]notebook\[Breve]title,
         WindowSize -> All,
         WindowMargins -> {{Automatic, 0}, {Automatic, 500}},
         Saveable -> False,
         WindowElements -> {"StatusArea", "MemoryMonitor",
            "MagnificationPopUp", "MenuBar"}];
   If[$VersionNumber>=7,
      Goto\[Breve]Start\[Breve]Box[conditionals\[Breve]notebook],
         SelectionMove[conditionals\[Breve]notebook, Before,Notebook];
      SelectionMove[conditionals\[Breve]notebook, Next, Cell, 3(*+Length[all\[Breve]conditionals]*)];
      SelectionMove[conditionals\[Breve]notebook, Before, CellContents]; 
      SelectionMove[conditionals\[Breve]notebook, Next,Character, 12]; 
      ];
   ];

drop\[Breve]conditional[n_]:=(
   cond\[Breve]expr="";
   all\[Breve]conditionals=Delete[all\[Breve]conditionals,n];
   );

add\[Breve]conditional[ex_]:=Module[{ex1="Hold["<>ex<>"]",expr},
   If[StringMatchQ[ex,WhitespaceCharacter...],Return[]];
   If[! SyntaxQ[ex1],
      CreateDialog[{"Conditional expression - invalid syntax",
            DefaultButton[]}];
      Return[];
      ];
   expr=ToExpression[ex1];
   If[! FreeQ[expr,Set],
      CreateDialog[{"Conditional expression must not contain '=' \nPerhaps you meant '=='?",
            DefaultButton[]}];
      Return[];
      ];
   all\[Breve]conditionals=Append[all\[Breve]conditionals,expr/.Hold->HoldForm];
   cond\[Breve]expr="";
   ];


get\[Breve]paleete\[Breve]nb[]:=Module[{ans},
   ans=Select[Notebooks[],(Window\[Breve]Title[#]==main\[Breve]palette\[Breve]title)&];
   If[ans==={},Null,First[ans]]
   ];


set\[Breve]status\[Breve]line[]:=Module[{nb=get\[Breve]paleete\[Breve]nb[]},
   If[NumberQ[step\[Breve]counter],
      stepping\[Breve]info="Step: "<>ToString[step\[Breve]counter],
         stepping\[Breve]info="Step: "<>ToString[0];
      ];
   ];


the\[Breve]time[]:=Module[{ans},
   ans=Drop[DateList[],3];
   ans[[3]]=0.1*Round[10*ans[[3]]];
   ToString[StringForm["``:``:``",Sequence@@ans]]
   ];


pause\[Breve]time=-1;
occasional\[Breve]pause[]:=If[
   pause\[Breve]time+1<AbsoluteTime[],
      pause\[Breve]time=AbsoluteTime[];
   Pause[0.1];
   set\[Breve]status\[Breve]line[];
   ];


Options[Debug\[Breve]Tag]={"ExtraInfo"->""};
Debug\[Breve]Tag[ll_,info_,opts:OptionsPattern[Debug\[Breve]Tag]]:=Module[{ex},
   If[in\[Breve]dialog || !debugging,Return[]];
   If[$DynamicEvaluation,
      If[dynamic\[Breve]debug\[Breve]count>0,Dynamic\[Breve]Debug\[Breve]Tag[ll,info]],
         last\[Breve]line\[Breve]encountered=ll;
      (*If[MathLink`IsPreemptive[],Return[]];*)
      step\[Breve]counter++;
      (* this is necessary to let palette work! *)
      (*If[Mod[step\[Breve]counter,100]==0,Pause[0.1]];*)
      occasional\[Breve]pause[];
      active\[Breve]breakpoints=True;
      If[active\[Breve]breakpoints && breakpoint\[Breve]id[ll]==ACTIVE\[Breve]BP && replacement\[Breve]message===Null,
         Print\[Breve]Debug\[Breve]Info[ll,"Stopping at breakpoint "];
         Print\[Breve]Attached\[Breve]Expressions[ll];
         step\[Breve]at=\[Infinity];
         stepping=True;
         Perform\[Breve]Dialog\[Breve]Pause[ll];
         active\[Breve]breakpoints=False;
         Return[],
            If[all\[Breve]conditionals=!={} && Or@@all\[Breve]conditionals/.HoldForm[x_]->x,
               Print\[Breve]Debug\[Breve]Info[ll,"Stopping at conditional breakpoint "];
            step\[Breve]at=\[Infinity];
            stepping=True;
            Perform\[Breve]Dialog\[Breve]Pause[ll];
            active\[Breve]breakpoints=False;
            Return[];
            ];
         ];
      ex=ToString[OptionValue["ExtraInfo"]];
      If[(stepping) || step\[Breve]counter>= step\[Breve]at,
         If[step\[Breve]at<\[Infinity],stepping=True];
         step\[Breve]at=\[Infinity];
         If[replacement\[Breve]message===Null,
            Print\[Breve]Debug\[Breve]Info[ll,info<>ex],
               Print\[Breve]Debug\[Breve]Info[ll,replacement\[Breve]message<>ex];
            replacement\[Breve]message=Null;
            ];
         Perform\[Breve]Dialog\[Breve]Pause[ll],
            If[ print\[Breve]steps\[Breve]regardless,Print\[Breve]Debug\[Breve]Info[ll,info<>ex]] 
         ];
      ];
   ];


Print\[Breve]Attached\[Breve]Expressions[lno_]:=Module[{llex},
   llex=bp\[Breve]breaks[lno];
   If[Head[llex]===List,
      While[llex=!={},
         Print\[Breve]Debug\[Breve]Info[lno,ToString[HoldForm@@Evaluate@First[llex]]<>"="<>ToString[ReleaseHold[First[llex]]]];
         llex=Drop[llex,1];
         ]
      ];
   ];


Options[Dynamic\[Breve]Debug\[Breve]Tag]={"ExtraInfo"->""};
Dynamic\[Breve]Debug\[Breve]Tag[ll_,info_,opts:OptionsPattern[Dynamic\[Breve]Debug\[Breve]Tag]]:=Module[{ex,llex,done=False},
   last\[Breve]line\[Breve]encountered=ll;
   If[in\[Breve]dialog || !debugging,Return[]];
   step\[Breve]counter++;
   (* occasional\[Breve]pause[];*)
   If[breakpoint\[Breve]id[ll]==ACTIVE\[Breve]BP,
      Print\[Breve]Debug\[Breve]Info[ll,"Passing breakpoint "];
      Print\[Breve]Attached\[Breve]Expressions[ll];
      done=True,
         If[all\[Breve]conditionals=!={} && Or@@all\[Breve]conditionals/.HoldForm[x_]->x,
            Print\[Breve]Debug\[Breve]Info[ll,"Passing conditional breakpoint "];
         done=True;
         ];
      ];
   If[processing\[Breve]message,
      Print\[Breve]Debug\[Breve]Info[SequenceForm[ll,", ",latest\[Breve]diagnostic\[Breve]message],"Diagnostic message "];
      Print\[Breve]Trace[];
      done=True;
      ];
   If[done,
      dynamic\[Breve]debug\[Breve]count--;
      If[dynamic\[Breve]debug\[Breve]count<=0,ResetDebugger[]];
      ];
   ];


Perform\[Breve]Dialog\[Breve]Pause[ll_]:=Module[{now},
   (* NotebookFind[source\[Breve]nb,line\[Breve]number\[Breve]string[ll]];*)
   now=AbsoluteTime[];
   set\[Breve]status\[Breve]line[];
   update\[Breve]breakpoint\[Breve]locations[];
   SelectionMove[source\[Breve]nb,CellContents,Before];
   brselect@breakpoint\[Breve]id[ll];re\[Breve]colour\[Breve]line[ll];
   If[SelectedNotebook[]=!=conditionals\[Breve]notebook,SetSelectedNotebook[notes\[Breve]nb]];
   in\[Breve]dialog=True;
   stepping\[Breve]over=False;
   print\[Breve]steps\[Breve]regardless=False;
   task=Null;
   status\[Breve]info="PAUSED";
   If[one\[Breve]shot\[Breve]line>0,
      brswap@breakpoint\[Breve]id[one\[Breve]shot\[Breve]line];
      re\[Breve]colour\[Breve]line[one\[Breve]shot\[Breve]line];
      one\[Breve]shot\[Breve]line=-1;
      ];
   (*\[LightBulb]1*)
   If[step\[Breve]timing && time\[Breve]at\[Breve]step\[Breve]start>0,
      print\[Breve]db[Style[SequenceForm["(",SetAccuracy[now-time\[Breve]at\[Breve]step\[Breve]start,2], " secs)"],Blue]];
      ];
   Block[{$ContextPath},
   If[!MemberQ[$ContextPath,"Global`"],
     print\[Breve]db["*****Warning Global` not on path"];
   ];
   $ContextPath=Append[$ContextPath,"DebugTrace`"];
   Dialog[];  
   ];  
   in\[Breve]dialog=False;
   Switch[task,
      task\[Breve]abort,
         Perform\[Breve]Abort[],

      task\[Breve]trace,
         Print\[Breve]Trace[];
      Perform\[Breve]Dialog\[Breve]Pause[ll],

      task\[Breve]adjust\[Breve]breakpoints,
         Adjust\[Breve]Breakpoints[];
      Perform\[Breve]Dialog\[Breve]Pause[ll],

      task\[Breve]conditionals,
         Adjust\[Breve]Conditionals[];
      Perform\[Breve]Dialog\[Breve]Pause[ll],

      task\[Breve]step\[Breve]over,
         stepping\[Breve]over=True;
      stepping=True;
      status\[Breve]info="RUNNING",
         Null,
         status\[Breve]info="RUNNING"
      ];
   If[SelectedNotebook[]=!=conditionals\[Breve]notebook,SetSelectedNotebook[notes\[Breve]nb]];
   brdeselect@breakpoint\[Breve]id[ll];re\[Breve]colour\[Breve]line[ll];
   time\[Breve]at\[Breve]step\[Breve]start=AbsoluteTime[];
   ];

Print\[Breve]Debug\[Breve]Info[ll_,info_]:=Module[{extra},
   (*last\[Breve]line=ll;*)
   extra="";
   If[StringLength[info]>0,extra=" ("<>info<>")"];
   print\[Breve]db[Style[SequenceForm["Line ",ll,extra," (Step ",step\[Breve]counter,")"],Blue]]   
   ];


SetAttributes[Function\[Breve]Tag,HoldFirst];
Function\[Breve]Tag[fn_,fname_,ln_]:=Block[{stack\[Breve]base={stack\[Breve]base,"function",fname,ln,last\[Breve]line\[Breve]encountered,$ModuleNumber},ans,tl},
   If[in\[Breve]dialog || ! debugging,Return[fn]];
   tl=last\[Breve]line\[Breve]encountered;
   If[stepping\[Breve]over,
      stepping\[Breve]over=False;
      stepping=False;     
      Debug\[Breve]Tag[ln,"Entering function "<>fname];
      ans=fn;
      Debug\[Breve]Tag[ln,"Leaving function "<>fname];
      stepping\[Breve]over=True;
      stepping=True,

      Debug\[Breve]Tag[ln,"Entering function "<>fname];
      ans=fn;
      Debug\[Breve]Tag[last\[Breve]line\[Breve]encountered,"Leaving function "<>fname]; 
      ];
   last\[Breve]line\[Breve]encountered=tl;
   ans
   ];

SetAttributes[Delayed\[Breve]variable\[Breve]Tag,HoldFirst];
Delayed\[Breve]variable\[Breve]Tag[var_,varname_,ln_]:=Block[{stack\[Breve]base={stack\[Breve]base,"delayed variable evaluation",varname,ln,last\[Breve]line\[Breve]encountered},ans,tl},
   If[in\[Breve]dialog || ! debugging,Return[var]];
   tl=last\[Breve]line\[Breve]encountered;
   Debug\[Breve]Tag[ln,"Entering delayed evaluation of variable "<>varname];
   ans=var;
   Debug\[Breve]Tag[ln,"Completing delayed evaluation of "<>varname];
   last\[Breve]line\[Breve]encountered=tl;
   ans
   ];


(*SyntaxInformation[DTDebugExecute]={"ArgumentsPattern"->{_,OptionsPattern[]}};*)
SetAttributes[DTDebugExecute,HoldFirst];
Options[DTDebugExecute]={};
DTDebugExecute[command_,OptionsPattern[DTDebugExecute]]:=Module[{ans},
   If[debugging,
      Print[Style["****You must quit the current debug session before starting another *****",Purple]];
      Return[];
      ];
   Reset\[Breve]Debugger[];
   debugger\[Breve]loaded=True;
   stepping=True;
   stepping\[Breve]over=False;
   message\[Breve]interception=True;
   step\[Breve]counter=0;
   step\[Breve]at=\[Infinity];  
   stack\[Breve]base=Null;
   debugging=True;
   ans=Catch[
      tmp=command;
      Print[Style["****Code execution completed****",Purple]];
      tmp,
         debug\[Breve]abort\[Breve]tag
      ];
   Reset\[Breve]Debugger[];
   ans
   ];



debug\[Breve]dynamics[]:=Module[{},   
   If[dynamic\[Breve]debug\[Breve]count>0,
      dynamic\[Breve]debug\[Breve]count=0;
      Reset\[Breve]Debugger[],    
         If[status\[Breve]info=="INACTIVE",
            Reset\[Breve]Debugger[];
         stepping=True;
         stepping\[Breve]over=False;
         message\[Breve]interception=True;
         step\[Breve]counter=0;
         step\[Breve]at=\[Infinity];  
         stack\[Breve]base=Null;
         debugging=True;
         dynamic\[Breve]results={};
         dynamic\[Breve]debug\[Breve]count=multi\[Breve]step;  
         message\[Breve]interception=message\[Breve]interception\[Breve]required;
         ];
      ]; 
   ];


(* ::Section:: *)
(*User callable functions*)


(*SyntaxInformation[DTDynamicResults]={"ArgumentsPattern"->{OptionsPattern[]}};*)
Options[DTDynamicResults]={"Keep"->False};
DTDynamicResults[OptionsPattern[DTDynamicResults]]:=Module[{ans},
   ans=Flatten[dynamic\[Breve]results];
   If[! OptionValue["Keep"],dynamic\[Breve]results={}];
   If[dynamic\[Breve]debug\[Breve]count>0,
      dynamic\[Breve]debug\[Breve]count=0;
      Reset\[Breve]Debugger[]
      ];
   ans
   ];


print\[Breve]dynamics[]:=Module[{k,len},
   k=1;
   len=Length[dynamic\[Breve]results];
   While[k <= len,
      NotebookWrite[notes\[Breve]nb,Cell[BoxData[dynamic\[Breve]results[[k]]],"Print"]];
      k++;
      ]
   ];



(*SyntaxInformation[DTLocalValue]={"ArgumentsPattern"->{_Symbol}};*)
SetAttributes[DTLocalValue,HoldFirst];
DTLocalValue[k_]:=Module[{lst,str},
   str=ToString[Unevaluated[k]];
   lst=Names[str<>"$*"];
   strpat=(((str<>"$")~~NumberString) |((str<>"$DebugTrace$")~~NumberString));
   lst=Select[lst,StringMatchQ[#,strpat]&];
   (*lst=Join[Names[(ToString[k]<>"$")~~NumberString],Names[(ToString[k]<>"$DebugTrace$")~~NumberString]];*)
   If[lst==={},Return[$Unknown]];
   lst=Map[{#,ToExpression[Last[StringSplit[#,"$"]]]}&,lst];
   ToExpression[Sort[lst,(#1[[2]]>#2[[2]])&][[1,1]]]
   ];


(*SyntaxInformation[DTLocalSymbols]={"ArgumentsPattern"->{_Symbol}};*)
SetAttributes[DTLocalSymbols,HoldFirst];
DTLocalSymbols[k_]:=Module[{lst,str,j,tmp,sb},
   str=ToString[Unevaluated[k]];
   lst=Names[str<>"$*"];
   strpat=(((str<>"$")~~NumberString) |((str<>"$DebugTrace$")~~NumberString));
   lst=Select[lst,StringMatchQ[#,strpat]&];
   (*lst=Join[Names[(ToString[k]<>"$")~~NumberString],Names[(ToString[k]<>"$DebugTrace$")~~NumberString]];*)
   If[lst==={},
      Print["******** No symbols found *******"];
      Return[Null];
      ];
   lst=Map[{#,ToExpression[Last[StringSplit[#,"$"]]]}&,lst];
   lst=Sort[lst,(#1[[2]]>#2[[2]])&];
   j=1;
   While[j <= Length[lst],
      tmp=lst[[j]];
      sb=stack\[Breve]base;
      While[sb=!={},
         If[sb[[2]]=="function" && sb[[6]]<tmp[[2]],
            Print["Variable ",tmp[[1]]," within execution of function ",sb[[3]]];
            Break[];        
            ];
         sb=sb[[1]];
         ];
      If[sb==={},Print[tmp[[1]]]];
      j++;
      ];
   ];


SetAttributes[DTPrint,HoldFirst];
Options[DTPrint]={"RHSOnly"->False};
DTPrint[x_,OptionsPattern[DTPrint]]:=Module[{ans=x},
   CellPrint[Cell[BoxData[
            ToBoxes[If[OptionValue["RHSOnly"]===True||Unevaluated[x]===ans,ans,SequenceForm[Defer[Short[x]],"=",Short[ans,60]]]]
            ],
            Background->RGBColor[0.6`,1,0.6`],"Print"]];
   ans
   ];


(* ::Section::Closed:: *)
(*Cleanup notebooks from possible previous run*)


clear\[Breve]source[];
clear\[Breve]palette[];
clear\[Breve]conditionals[];


(*Print["Loaded DebugTrace version ",version," \[Dash] for help, execute DebugTraceHelp[]"];*)


(* ::Section::Closed:: *)
(*End of package*)


End[];


EndPackage[] 
