(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     97248,       2299]
NotebookOptionsPosition[     89903,       2080]
NotebookOutlinePosition[     91085,       2120]
CellTagsIndexPosition[     91042,       2117]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["DebugTrace", "Chapter",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.449466398453125*^9, 3.4494664025625*^9}}],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}}],

Cell[TextData[{
 "Since version 6.0, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has come with a built in debugger. The DebugTrace package is an attempt to \
provide a more fully featured and extensible debugger, tailored particularly \
at the beginner user (who is most likely to need its help!). In writing this \
package, I have thought hard about what features are important in a \
Mathematica debugger. Here are a few general principles:\n\n\
\[FilledSmallCircle] Most people only turn to a debugger as a last resort. \
When something is not working, the last thing they want is to open another \
large manual!\n\[FilledSmallCircle] A debugger for any language is of \
particular value to beginners. Unfortunately, many novice users write their \
code at the top level (no user-defined functions), and run it by simply \
executing their whole notebook. A debugger absolutely must cater for (but not \
be limited to) that style of working.\n\[FilledSmallCircle] Those of us who \
style ourselves as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Consultants, like to create code that is nicely structured into functions, \
but we spend much of our time exploring code that we did not write, and which \
is often written in a very different style! A debugger can really help when \
presented with thousands of lines of such code.\n\[FilledSmallCircle] \
Debuggers for most languages depend heavily on line numbers, whereas because \
of the nature of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " language, the built-in debugger highlights where you are in the code. The \
problem is that as you step the code, the selection moves, and nothing \
remains to examine later \[Dash] for large problems, this approach is very \
ineffective. By contrast, DebugTrace uses line numbers throughout. This means \
that if you write all your code on one line, DebugTrace will be of no help to \
you! By laying out your code, you give DebugTrace line numbers to enable it \
to report to you. Conversely, if you are not interested in debugging certain \
bits of a program, condensing them on to one line is one way (not really \
recommended!) to make DebugTrace ignore them.\n\[FilledSmallCircle] All the \
information supplied by DebugTrace is interleaved with other output from your \
program, and remains in your notebook for you to examine later. This is of \
particular value when debugging larger programs.\n\[FilledSmallCircle] When \
debugging a complex program, most people need to use a debugger more than \
once \[Dash] relating the output from one run with that from the next. \
DebugTrace gives every step (whether shown or not) a step number. This means \
that if you establish that your program breaks down at step 5023, you may \
find it useful to re-run it and break it at (say) step 4900, and just \
single-step it to explore what is going on.\n\[FilledSmallCircle] Because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is interactive, and you can execute any operation while paused at a \
breakpoint, even a simple debugger can be remarkably effective \[Dash] you \
don't need a large number of complex commands, which are hard to remember and \
use on the odd occasions when you need to debug code."
}], "Text",
 CellChangeTimes->{{3.442235566370158*^9, 3.44223556992527*^9}, {
  3.442235600779637*^9, 3.442235751997077*^9}, {3.442650330609375*^9, 
  3.4426503473125*^9}, {3.442650395515625*^9, 3.442651451078125*^9}, {
  3.44265162371875*^9, 3.44265207671875*^9}, {3.442761851984375*^9, 
  3.4427618555625*^9}, {3.4427618981875*^9, 3.442761924328125*^9}, {
  3.44276196853125*^9, 3.442761976984375*^9}, {3.442762015703125*^9, 
  3.442762073609375*^9}, {3.442954667046875*^9, 3.442954669859375*^9}, {
  3.44295472609375*^9, 3.442954777203125*^9}, {3.4430382805625*^9, 
  3.44303828878125*^9}, {3.445265511296875*^9, 3.445265515125*^9}, {
  3.44569539990625*^9, 3.4456954025*^9}, {3.446101909078125*^9, 
  3.446101917203125*^9}, {3.449069021125*^9, 3.449069044234375*^9}, {
  3.449084240703125*^9, 3.44908429309375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing the package", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.443038433046875*^9, 3.44303844053125*^9}}],

Cell[TextData[{
 "DebugTrace is supplied as a ZIP file containing all the files required in \
their appropriate directories.. It is vital that this directory structure is \
preserved. Copy the file either to the directory specified by $BaseDirectory, \
or $UserBaseDirectory (as a general rule, use the former if you are the sole \
user of the computer, use the latter if several people log on to your \
machine). ",
 StyleBox["Do not use the ",
  FontVariations->{"Underline"->True}],
 StyleBox["Mathematica",
  FontSlant->"Italic",
  FontVariations->{"Underline"->True}],
 StyleBox[" directory itself.",
  FontVariations->{"Underline"->True}],
 "  Unzip the file DebugTrace.ZIP using a tool that preserves the directory \
structure and handles long names correctly, e.g. PKZIP(R)  Version 2.50, or \
WinZip(R). ",
 StyleBox["Never add or remove anything from the DebugTrace directory or any \
of its subdirectories",
  FontVariations->{"Underline"->True}],
 ", as this can disturb the operation of the package."
}], "Text",
 CellChangeTimes->{
  3.410454814007333*^9, {3.44303844590625*^9, 3.44303844846875*^9}, {
   3.44303848046875*^9, 3.44303851453125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Getting started", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}}],

Cell["\<\
Before applying the debugger to your own code, I strongly recommend that you \
spend a few minutes using the toy example supplied. \
\>", "Text",
 CellChangeTimes->{{3.442652182328125*^9, 3.44265239128125*^9}, {
   3.442762517875*^9, 3.442762553046875*^9}, {3.44276259021875*^9, 
   3.44276263453125*^9}, {3.442951219328125*^9, 3.442951223046875*^9}, {
   3.446047881703125*^9, 3.44604794759375*^9}, 3.44610196925*^9}],

Cell[TextData[{
 "Copy the notebook \"ToyExample.nb\" to a location of your own choosing \
\[Dash] say c:\\test\\ToyExample.nb \[Dash] and start ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from scratch, ",
 StyleBox["taking care to ensure that the built-in ",
  FontColor->RGBColor[1, 0, 0]],
 StyleBox["Mathematica",
  FontSlant->"Italic",
  FontColor->RGBColor[1, 0, 0]],
 StyleBox[" debugger is turned off",
  FontColor->RGBColor[1, 0, 0]],
 ". Then open the notebook ToyExample.nb and open the palette entitled \
\"DebugTrace control panel\". Use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 'Palette' menu to do this if this palette is not already open."
}], "Text",
 CellChangeTimes->{{3.442652182328125*^9, 3.44265239128125*^9}, {
  3.442762517875*^9, 3.442762553046875*^9}, {3.44276259021875*^9, 
  3.44276263453125*^9}, {3.442951219328125*^9, 3.442951223046875*^9}, {
  3.446047881703125*^9, 3.44604794759375*^9}, {3.44610196925*^9, 
  3.44610197503125*^9}, {3.449084352515625*^9, 3.449084388375*^9}}],

Cell[TextData[StyleBox["Please note that the toy example is very definitely \
not written in a clever or efficient way, and contains a deliberate mistake!",
 FontColor->RGBColor[1, 0, 0]]], "Text",
 CellChangeTimes->{{3.44276215859375*^9, 3.44276221921875*^9}, {
  3.442951251015625*^9, 3.442951254765625*^9}}],

Cell["\<\
Click on the ToyExample notebook to ensure it is selected, and then press the \
red \"Debug this notebook\" button. Wait until a new notebook entitled \
\"DebugTrace : Results\" has been created. Although this should happen \
rapidly for the toy example, this step may take several seconds for larger \
programs.\
\>", "Text",
 CellChangeTimes->{{3.44604799453125*^9, 3.44604807953125*^9}, {
  3.446048199125*^9, 3.44604823659375*^9}, {3.446048275171875*^9, 
  3.446048341046875*^9}}],

Cell["\<\
Note that when you work on your own code, this can be stored in either a \
notebook (.nb) or package (.m) file. To test a properly structured package, \
one approach is to take a copy of the file, and add one or more test function \
calls at the end of the file (typically after the call to EndPackage[] ).

It is also possible to select a number of cells in a notebook and use \
DebugTrace on these cells using the \"Debug selection\" button. This button \
will only operate on selections at the cell level \[Dash] typically made by \
dragging the mouse over one or more cell brackets.\
\>", "Text",
 CellChangeTimes->{{3.442652479328125*^9, 3.442652559375*^9}, {
  3.442762690203125*^9, 3.442762832015625*^9}, {3.4460481104375*^9, 
  3.446048125828125*^9}, {3.44610209671875*^9, 3.446102101046875*^9}, {
  3.45622955965625*^9, 3.456229691875*^9}}],

Cell["\<\
At this point, you should also see a second copy of the code annotated with \
line numbers. Each line number is actually a button \[Dash] as you will see \
shortly. Thus you should now have three notebooks open \[Dash] the notebook \
which you want to debug, a copy of this notebook arranged with line numbers \
down the left hand side, and the above-mentioned results notebook. In \
addition you have the DebugTrace control panel palette open. You will find a \
blue message in the results notebook telling you where your program is \
positioned - at the first executable line.\
\>", "Text",
 CellChangeTimes->{{3.44265257321875*^9, 3.442652676609375*^9}, {
   3.442762302640625*^9, 3.442762395984375*^9}, {3.442762443453125*^9, 
   3.442762502578125*^9}, {3.4450991778125*^9, 3.4450991815625*^9}, {
   3.44604837603125*^9, 3.446048396296875*^9}, {3.446048439515625*^9, 
   3.44604845375*^9}, {3.446102138484375*^9, 3.4461021885*^9}, {
   3.44610223746875*^9, 3.44610236275*^9}, 3.449084410703125*^9}],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "1", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "1", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 1, "", " (Step ", 1, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019532125*^9},
 CellLabel->"During evaluation of In[2]:="],

Cell["\<\
The toy example calculates Legendre polynomials from the 3-term recurrence \
relation. It's only error is that the list 'output' is not long enough to \
record the last result. Because this program is so short, it is reasonable to \
print every step. Using the palette, press \"Message interception\" to turn \
that feature off, and press \"Step once\" a few times to watch the debugger \
stepping through the code. Now press \"Print N steps\".\
\>", "Text",
 CellChangeTimes->{{3.4429512775*^9, 3.4429515363125*^9}, {3.443020311125*^9, 
  3.443020373140625*^9}}],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "1", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "1", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 1, "", " (Step ", 1, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019532125*^9},
 CellLabel->"During evaluation of In[2]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "2", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "2", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 2, "", " (Step ", 2, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "11", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "3", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 11, "", " (Step ", 3, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953803125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "2", 
    "\[InvisibleSpace]", "\<\" (Entering function CalculateLegendre)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "4", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm[
   "Line ", 2, " (Entering function CalculateLegendre)", " (Step ", 4, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195380625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "2", 
    "\[InvisibleSpace]", "\<\" (In Module with local variables {n$1406})\"\>",
     "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm[
   "Line ", 2, " (In Module with local variables {n$1406})", " (Step ", 5, 
    ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538109375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "3", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "6", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 3, "", " (Step ", 6, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538140625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "4", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "7", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 4, "", " (Step ", 7, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538171875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 8, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953821875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "9", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 9, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953825*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "10", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 10, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953828125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "11", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 11, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195383125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "12", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 12, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538359375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "13", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 13, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538546875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "14", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 14, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538578125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "15", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 15, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538609375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "16", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 16, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953865625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "17", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 17, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195386875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "18", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 18, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953871875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "19", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 19, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "20", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 20, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953878125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "21", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 21, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195388125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "22", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 22, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953884375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "23", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 23, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538890625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "24", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 24, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538921875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "25", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 25, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538953125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "26", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 26, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019538984375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "27", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 27, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539015625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "28", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 28, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539046875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "29", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 29, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539078125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "30", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 30, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539109375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "31", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 31, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539140625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\" (Executing body of For)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "32", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 5, " (Executing body of For)", " (Step ", 32, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195391875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "33", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 7, "", " (Step ", 33, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953921875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "34", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, "", " (Step ", 34, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953925*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Set", "::", "\<\"partw\"\>"}], ":", 
  " ", "\<\"Part \\!\\(10\\) of \\!\\({0, \\(\\(\[LeftSkeleton] 7 \
\[RightSkeleton]\\)\\), \\(\\(\\(315\\\\ x\\)\\/128 - \\(1155\\\\ \
x\\^3\\)\\/32 + \\(9009\\\\ x\\^5\\)\\/64 - \\(6435\\\\ x\\^7\\)\\/32 + \
\\(12155\\\\ x\\^9\\)\\/128\\)\\)}\\) does not exist. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partw\\\", ButtonNote -> \
\\\"Set::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.4430195105625*^9, 3.443019539375*^9}},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "2", 
    "\[InvisibleSpace]", "\<\" (Leaving function CalculateLegendre)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "35", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm[
   "Line ", 2, " (Leaving function CalculateLegendre)", " (Step ", 35, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539421875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "12", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "36", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 12, "", " (Step ", 36, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539453125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "2"]}], "+", 
    FractionBox[
     RowBox[{"3", " ", 
      SuperscriptBox["x", "2"]}], "2"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"3", " ", "x"}], "2"]}], "+", 
    FractionBox[
     RowBox[{"5", " ", 
      SuperscriptBox["x", "3"]}], "2"]}], ",", 
   RowBox[{
    FractionBox["3", "8"], "-", 
    FractionBox[
     RowBox[{"15", " ", 
      SuperscriptBox["x", "2"]}], "4"], "+", 
    FractionBox[
     RowBox[{"35", " ", 
      SuperscriptBox["x", "4"]}], "8"]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"15", " ", "x"}], "8"], "-", 
    FractionBox[
     RowBox[{"35", " ", 
      SuperscriptBox["x", "3"]}], "4"], "+", 
    FractionBox[
     RowBox[{"63", " ", 
      SuperscriptBox["x", "5"]}], "8"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox["5", "16"]}], "+", 
    FractionBox[
     RowBox[{"105", " ", 
      SuperscriptBox["x", "2"]}], "16"], "-", 
    FractionBox[
     RowBox[{"315", " ", 
      SuperscriptBox["x", "4"]}], "16"], "+", 
    FractionBox[
     RowBox[{"231", " ", 
      SuperscriptBox["x", "6"]}], "16"]}], ",", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"35", " ", "x"}], "16"]}], "+", 
    FractionBox[
     RowBox[{"315", " ", 
      SuperscriptBox["x", "3"]}], "16"], "-", 
    FractionBox[
     RowBox[{"693", " ", 
      SuperscriptBox["x", "5"]}], "16"], "+", 
    FractionBox[
     RowBox[{"429", " ", 
      SuperscriptBox["x", "7"]}], "16"]}], ",", 
   RowBox[{
    FractionBox["35", "128"], "-", 
    FractionBox[
     RowBox[{"315", " ", 
      SuperscriptBox["x", "2"]}], "32"], "+", 
    FractionBox[
     RowBox[{"3465", " ", 
      SuperscriptBox["x", "4"]}], "64"], "-", 
    FractionBox[
     RowBox[{"3003", " ", 
      SuperscriptBox["x", "6"]}], "32"], "+", 
    FractionBox[
     RowBox[{"6435", " ", 
      SuperscriptBox["x", "8"]}], "128"]}], ",", 
   RowBox[{
    FractionBox[
     RowBox[{"315", " ", "x"}], "128"], "-", 
    FractionBox[
     RowBox[{"1155", " ", 
      SuperscriptBox["x", "3"]}], "32"], "+", 
    FractionBox[
     RowBox[{"9009", " ", 
      SuperscriptBox["x", "5"]}], "64"], "-", 
    FractionBox[
     RowBox[{"6435", " ", 
      SuperscriptBox["x", "7"]}], "32"], "+", 
    FractionBox[
     RowBox[{"12155", " ", 
      SuperscriptBox["x", "9"]}], "128"]}]}], "}"}]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953946875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "13", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "37", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 13, "", " (Step ", 37, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.4430195395*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"-", 
    FractionBox["1", "2"]}], ",", "0", ",", 
   FractionBox["3", "8"], ",", "0", ",", 
   RowBox[{"-", 
    FractionBox["5", "16"]}], ",", "0", ",", 
   FractionBox["35", "128"], ",", "0"}], "}"}]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.443019539515625*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox["\<\"****Code execution completed****\"\>",
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0.5, 0, 0.5],
  $CellContext`BackFaceColor -> RGBColor[0.5, 0, 0.5],
  $CellContext`GraphicsColor -> RGBColor[0.5, 0, 0.5],
  FontColor->RGBColor[0.5, 0, 0.5]]], "Print",
 CellChangeTimes->{3.4430194993125*^9, 3.44301953953125*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell["\<\
The default value of N is 100 steps \[Dash] which is longer than this \
program, so the program has completed. Press \"Set N\" to change this default \
value.\
\>", "Text",
 CellChangeTimes->{{3.44302039140625*^9, 3.443020460359375*^9}}],

Cell["\<\
The above printout contains a wealth of information about the program. Note \
that the line numbers refer to the source listing, and that the error \
occurred on line 7. The step numbers (not to be confused with the line \
numbers!)  are useful if we need to re-run the debugger on the same program. \
As you can see, ordinary output from the program (including the error \
message), is interspersed with debug information (printed using coloured \
text).\
\>", "Text",
 CellChangeTimes->{{3.4429525945625*^9, 3.442952673484375*^9}, {
  3.4429527040625*^9, 3.442952728984375*^9}, {3.44295287078125*^9, 
  3.442952921171875*^9}, {3.44295449515625*^9, 3.442954521875*^9}, {
  3.4430204933125*^9, 3.443020509109375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Killing the kernel", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.4490844949375*^9, 
  3.449084504484375*^9}}],

Cell[TextData[{
 "When you execute definitions in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", they effectively overlay existing definitions in the kernel. Usually this \
is what is required, but if you debug a flawed program and fix a bug, there \
is no guarantee that the kernel will recover from the mistake when you \
re-execute the code. Suppose, for example, you meant to write If[k==0,....., \
but you actually used the '=' operator rather than '==', this will have left \
a definition for k in the kernel. If in doubt, it is best to press the 'Kill \
kernel' button between debug sessions to start again with a clean slate. More \
advanced users, may obviously choose to do this only where necessary."
}], "Text",
 CellChangeTimes->{{3.449084535796875*^9, 3.44908497115625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Determining where an error happens", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}}],

Cell["\<\
Kill the kernel and re-run the debugger on the toy example, but this time \
leave \"Message interception\" enabled, and press \"Run Until break\", then \
press \"Trace\", followed by \"Quit\".\
\>", "Text",
 CellChangeTimes->{{3.4429528515625*^9, 3.442952861484375*^9}, {
  3.4429529318125*^9, 3.442953012515625*^9}, {3.442953319671875*^9, 
  3.44295332640625*^9}}],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "1", 
    "\[InvisibleSpace]", "\<\"\"\>", "\[InvisibleSpace]", "\<\" (Step \"\>", 
    "\[InvisibleSpace]", "1", "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 1, "", " (Step ", 1, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.443019385484375*^9},
 CellLabel->"During evaluation of In[2]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Set", "::", "\<\"partw\"\>"}], ":", 
  " ", "\<\"Part \\!\\(10\\) of \\!\\({0, \\(\\(\[LeftSkeleton] 7 \
\[RightSkeleton]\\)\\), \\(\\(\\(315\\\\ x\\)\\/128 - \\(1155\\\\ \
x\\^3\\)\\/32 + \\(9009\\\\ x\\^5\\)\\/64 - \\(6435\\\\ x\\^7\\)\\/32 + \
\\(12155\\\\ x\\^9\\)\\/128\\)\\)}\\) does not exist. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partw\\\", ButtonNote -> \
\\\"Set::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.443019395875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox[
   RowBox[{"\<\"Line \"\>", "\[InvisibleSpace]", "8", 
    "\[InvisibleSpace]", "\<\" (Break after message)\"\>", 
    "\[InvisibleSpace]", "\<\" (Step \"\>", "\[InvisibleSpace]", "35", 
    "\[InvisibleSpace]", "\<\")\"\>"}],
   SequenceForm["Line ", 8, " (Break after message)", " (Step ", 35, ")"],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 0, 1],
  $CellContext`BackFaceColor -> RGBColor[0, 0, 1],
  $CellContext`GraphicsColor -> RGBColor[0, 0, 1],
  FontColor->RGBColor[0, 0, 1]]], "Print",
 CellChangeTimes->{3.443019395921875*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox["\<\"In line \\!\\(8\\)\"\>",
   StringForm["In line ``", 8],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 
    Rational[2, 3], 0],
  $CellContext`BackFaceColor -> RGBColor[0, 
    Rational[2, 3], 0],
  $CellContext`GraphicsColor -> RGBColor[0, 
    Rational[2, 3], 0],
  FontColor->RGBColor[0, 
    Rational[2, 3], 0]]], "Print",
 CellChangeTimes->{3.443019402109375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox[
  InterpretationBox["\<\"In function CalculateLegendre that starts at line \
\\!\\(2\\) , called from line \\!\\(11\\)\"\>",
   StringForm[
   "In `` `` that starts at line `` , called from line ``", "function", 
    "CalculateLegendre", 2, 11],
   Editable->False],
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0, 
    Rational[2, 3], 0],
  $CellContext`BackFaceColor -> RGBColor[0, 
    Rational[2, 3], 0],
  $CellContext`GraphicsColor -> RGBColor[0, 
    Rational[2, 3], 0],
  FontColor->RGBColor[0, 
    Rational[2, 3], 0]]], "Print",
 CellChangeTimes->{3.443019402109375*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData[
 StyleBox["\<\"**** Quit by user ***\"\>",
  StripOnInput->False,
  $CellContext`FrontFaceColor -> RGBColor[0.5, 0, 0.5],
  $CellContext`BackFaceColor -> RGBColor[0.5, 0, 0.5],
  $CellContext`GraphicsColor -> RGBColor[0.5, 0, 0.5],
  FontColor->RGBColor[0.5, 0, 0.5]]], "Print",
 CellChangeTimes->{3.44301940525*^9},
 CellLabel->"During evaluation of In[3]:="],

Cell[BoxData["\<\"\"\>"], "Output",
 CellChangeTimes->{3.44301940525*^9},
 CellLabel->"Out[2]="],

Cell["\<\
Note that the program now runs normally (except a little less efficiently) \
until the error message is generated, when it prints the location and stops \
for user response. The (green) output from pressing the \"Trace\" button \
shows where the error occurred, which function was executing, and where it \
was called from \[Dash] tracing through as many levels as necessary, back to \
the original call (at line 11 in this case). 
When you are done debugging, do not leave the debugger active, abandon the \
debug session by pressing \"Quit\". \
\>", "Text",
 CellChangeTimes->{{3.4429532315*^9, 3.442953305375*^9}, {
  3.442953338953125*^9, 3.44295355746875*^9}, {3.446635726375*^9, 
  3.446635726875*^9}, {3.449084998734375*^9, 3.449085026703125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Performing side calculations", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
   3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
   3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
   3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
   3.4430196250625*^9}, 3.4430205540625*^9, {3.4430387565*^9, 
   3.443038765796875*^9}}],

Cell["\<\
Whenever the debugger is stopped at a break, you can do any side calculation \
you wish, except starting a new debug operation. If you want to refer to the \
local variables of a Module, either use the function DTLocalValue (see \
below), or note the actual names of the variables. For example, a \
module-local variable such as 'foo' will be renamed to a name such as \
'foo$430' \[Dash] different numbers are used each time a Module is executed. \
As you will see below, if you step into a Module, you will be told the actual \
names of its local variables.\
\>", "Text",
 CellChangeTimes->{{3.4430387835*^9, 3.443039033453125*^9}, {
  3.44303909290625*^9, 3.443039098921875*^9}, {3.443452303203125*^9, 
  3.443452331875*^9}, {3.4434524325*^9, 3.44345245390625*^9}, {
  3.44509922028125*^9, 3.44509923596875*^9}, {3.446635788625*^9, 
  3.4466357896875*^9}, {3.449085069859375*^9, 3.449085074390625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setting breakpoints", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
   3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
   3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
   3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
   3.4430196250625*^9}, 3.4430205540625*^9}],

Cell["\<\
Breakpoints are set by line number. Start the debugger on the toy example \
once more, and examine the window entitled \"DebugTrace: Source code\". You \
will see that the line number of each cell are either blue or grey. Grey line \
numbers cannot be selected as breakpoints for one reason or another. Blue \
line numbers indicate a location at which you can set a breakpoint. Click on \
line 11 to activate a breakpoint at that line \[Dash] the line at which you \
call the function. Now press \"Run until Break\" to advance to that line. \
Notice that along the way, you have supplied a definition for \
CalculateLegendre, and that as a result several lines inside that function \
have become available as potential breakpoints. For example, you might choose \
to click line 4 \[Dash] just before the loop. Once again, pressing \"Run \
until break\" will advance the program until it is at that line.\
\>", "Text",
 CellChangeTimes->{{3.443019746140625*^9, 3.44302026540625*^9}, 
   3.4466362233125*^9, 3.44908509353125*^9}],

Cell["\<\
The buttons can take several different colours, as indicated in the following \
table:\
\>", "Text",
 CellChangeTimes->{{3.446636779125*^9, 3.44663681196875*^9}}],

Cell[BoxData[GridBox[{
   {
    RowBox[{"BP", " ", "Available"}], 
    RowBox[{"BP", " ", "Set"}], 
    RowBox[{"Stopped", " ", "at", " ", "line"}], 
    RowBox[{"Button", " ", "colour"}]},
   {"No", "\[Dash]", "\[Dash]", "Grey"},
   {"Yes", "No", "No", "Blue"},
   {"Yes", "Yes", "No", "Red"},
   {"Yes", "No", "Yes", "Green"},
   {"Yes", "Yes", "Yes", "Orange"}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]], "Text",
 CellChangeTimes->{{3.44663660921875*^9, 3.44663676*^9}, {3.4490852118125*^9, 
  3.449085241984375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other ways to set breakpoints", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
   3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
   3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
   3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
   3.4430196250625*^9}, 3.4430205540625*^9, {3.5049677418125*^9, 
   3.504967751328125*^9}}],

Cell["\<\
If you simply want to run the program until it reaches a certain line, you \
can achieve this by clicking the line number in question while holding the \
CTRL key down.
Another useful way to force a breakpoint, is by actually calling a function \
from within the code. For example:\
\>", "Text",
 CellChangeTimes->{{3.504967760515625*^9, 3.50496785940625*^9}, {
   3.50496789359375*^9, 3.50496789928125*^9}, 3.50511859378125*^9}],

Cell[BoxData[
 RowBox[{"DTDebuggerInterrupt", "[", "\"\<Arrived somewhere interesting\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.504967863140625*^9, 3.504967889359375*^9}}],

Cell["\<\
Note that outside of DebugTrace, this function call will have no effect.\
\>", "Text",
 CellChangeTimes->{{3.504967906390625*^9, 3.504967930328125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Alternative ways to start debugging", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
   3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
   3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
   3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
   3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
   3.446048647078125*^9, 3.446048679078125*^9}, 3.446747874515625*^9}],

Cell["\<\
An earlier version of this software required the DebugTrace package to be \
loaded like any other package, and started by function calls. This style of \
operation is still available if desired:\
\>", "Text",
 CellChangeTimes->{{3.446048696609375*^9, 3.4460488165625*^9}}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<DebugTrace`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.442652399640625*^9, 3.442652411484375*^9}}],

Cell[BoxData[
 RowBox[{"DTRunDebugger", "[", "\"\<C:\\\\test\\\\ToyExample.nb\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.442652442859375*^9, 3.44265246171875*^9}, {
  3.445099163234375*^9, 3.445099167*^9}}],

Cell["\<\
Note that the DebugTrace` package is loaded when you press either the \
\"DebugTrace Help\" button or the \"Debug this notebook\" button.\
\>", "Text",
 CellChangeTimes->{{3.446048868078125*^9, 3.446048949625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Re-running the debugger", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}}],

Cell["\<\
A serious application of the debugger may require several executions of the \
same code. The easiest and safest way to re-run the debugger, is to kill the \
kernel, and start again. This ensures that the step numbers match up, and \
that changes made by one run of your code do not leave something that changes \
the behaviour on the second run (something you should, of course avoid!).\
\>", "Text",
 CellChangeTimes->{{3.443020597328125*^9, 3.44302081334375*^9}, 
   3.443038309375*^9}],

Cell["\<\
Alternatively, after you have loaded and run code using DTRunDebugger, you \
can execute another command using DTDebugExecute. For example, after running \
the toy example to completion, you could re-debug the function by calling:\
\>", "Text",
 CellChangeTimes->{{3.443020826109375*^9, 3.44302093896875*^9}, {
  3.44302099121875*^9, 3.4430209938125*^9}, {3.44509894228125*^9, 
  3.44509894696875*^9}, {3.445099245734375*^9, 3.445099250453125*^9}}],

Cell[BoxData[
 RowBox[{"DTDebugExecute", "[", 
  RowBox[{"CalculateLegendre", "[", "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.443020948828125*^9, 3.443020980859375*^9}, 
   3.44509896453125*^9}],

Cell["\<\
Note that the definitions in the notebook will not get re-executed, so the \
step numbers will be slightly smaller as you re-run the code. If you intend \
to use DTDebugExecute, it is probably best to use a notebook consisting of \
pure definitions, and supply the test cases to DTDebugExecute.\
\>", "Text",
 CellChangeTimes->{{3.443020999484375*^9, 3.44302101928125*^9}, {
  3.443021049515625*^9, 3.443021175078125*^9}, {3.44509897696875*^9, 
  3.44509898903125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Programs that get stuck", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}}],

Cell[TextData[{
 "It is all too easy to write code that runs for ever. This can happen for a \
variety of reasons, but pressing \"Interrupt\" will break into such loops, \
providing that the system is actually executing debugged code (it might get \
stuck in a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function, such as FullSimplify applied to a very large expression). If \
\"Interrupt\" works, try pressing \"Trace\" to obtain more information about \
where your program was looping.\nSometimes pressing \"Interrupt\" is \
ineffective \[Dash] typically for the reason described above \[Dash] or the \
program may fail for some other fatal reason that also kills the debugger, \
for example it may execute code such as this:\n"
}], "Text",
 CellChangeTimes->{{3.443021234671875*^9, 3.443021748171875*^9}, {
  3.443035352328125*^9, 3.443035441875*^9}, {3.4490861953125*^9, 
  3.4490861971875*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{"0", ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "1", ",", "1000000000"}], "}"}]}], "]"}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.443021774015625*^9, 3.4430217771875*^9}, {
   3.4430218085625*^9, 3.443021854234375*^9}, {3.443021899875*^9, 
   3.443021953*^9}, 3.443035285734375*^9},
 CellLabel->"In[11]:="],

Cell["\<\

No more memory available.
Mathematica kernel has shut down.
Try quitting other applications and then retry.\
\>", "Print",
 CellChangeTimes->{3.44303516584375*^9}]
}, Open  ]],

Cell["\<\
In this case, one approach is to experiment by running the program for N \
steps (perhaps with N increased) and determining just how far you can go \
before things break down. By zooming in to the point of failure on successive \
runs of the debugger, it is possible to determine where the problem occurs.\
\>", "Text",
 CellChangeTimes->{{3.443021234671875*^9, 3.4430217444375*^9}, {
  3.4430353161875*^9, 3.44303531678125*^9}}],

Cell["\<\
If done systematically, it is possible to zoom in on the point of failure \
very efficiently using a binary chop. Thus, if the program pauses OK at step \
500, but never reaches step 1000, try running to step 750 \[Dash] each time \
splitting the difference between the last point reached, and the lowest point \
where the program has failed. Even very obscure and difficult bugs can be \
narrowed down in this way.\
\>", "Text",
 CellChangeTimes->{{3.449086238859375*^9, 3.4490864545*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Local and argument variables", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.443449658875*^9, 3.443449671328125*^9}, {
  3.446637651359375*^9, 3.446637665109375*^9}}],

Cell[TextData[{
 "Consider the Module that comprises the body of CalculateLegendre in the toy \
example. The variable 'n' is local to the Module, which means that it will be \
replaced by a unique name such as n$451 each time the Module is executed. \
This is the standard way ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " operates, but it can cause confusion while debugging. If, for example, you \
step to line 8, you will find that the variable 'n' does not have a value \
(unless you gave it a global value). To reference the topmost local value, \
use the function DTLocalValue \[Dash] e.g. DTLocalValue[n]. If DTLocalValue \
cannot find a local value to return, it will return $Unknown.\n"
}], "Text",
 CellChangeTimes->{{3.4434497134375*^9, 3.443449716390625*^9}, {
   3.44344975278125*^9, 3.4434497914375*^9}, {3.44345064796875*^9, 
   3.4434509895625*^9}, {3.443451124765625*^9, 3.443451213453125*^9}, {
   3.443451307421875*^9, 3.44345131040625*^9}, {3.444159731375*^9, 
   3.444159750921875*^9}, {3.444159955265625*^9, 3.44415997128125*^9}, {
   3.445099016578125*^9, 3.4450990540625*^9}, 3.44908648034375*^9}],

Cell[TextData[{
 StyleBox["Never",
  FontVariations->{"Underline"->True}],
 ", use DTLocalValue in finished code \[Dash] the contents of the DebugTrace \
package should be used only for the purpose of debugging, or your code will \
become extremely obscure and inefficient!"
}], "Text",
 CellChangeTimes->{{3.443451271078125*^9, 3.443451303640625*^9}, {
   3.443454298546875*^9, 3.44345433634375*^9}, {3.443454374203125*^9, 
   3.443454377109375*^9}, 3.445099064421875*^9, {3.44509926125*^9, 
   3.445099271109375*^9}, {3.44908649309375*^9, 3.44908653459375*^9}, {
   3.44908658690625*^9, 3.449086660703125*^9}}],

Cell["\<\
In addition DTLocalValue also works on the pattern arguments of functions. \
Thus, if you have a function defined as myfunc[x_]:= ..... then if you break \
inside this function, you can use DTLocalValue[x] to determine the argument \
x. This is easy to remember, because syntax colouring will colour all such \
pattern variables green (by default) \[Dash] just like Module local \
variables. Pattern variables that match a sequence of arguments (e.g. x___) \
cannot be accessed currently.\
\>", "Text",
 CellChangeTimes->{{3.4466354436875*^9, 3.446635666546875*^9}, {
   3.446637673171875*^9, 3.446637727359375*^9}, {3.449086692875*^9, 
   3.449086694671875*^9}, 3.449508939651435*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Conditional breakpoints", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.44345135315625*^9, 3.4434513634375*^9}}],

Cell["\<\
Sometimes you need to break a program when some condition becomes True. \
Suppose, for example, you were still mystified as to why CalculateLegendre \
generated an error message. Debug the example from scratch once more, and at \
the first step, press the \"Conditionals\" button. Enter a test such as \
Last[output]=!=0 to test when the array fills up. Now press \"Run until break\
\", and you should find DebugTrace pauses just after the 9'th entry has been \
placed in the array 'output'.
You can have multiple conditional breakpoints active at one time. In order to \
fire, a condition must return True. It is often useful to use the \
DTLocalValue function in a conditional expression, for example:\
\>", "Text",
 CellChangeTimes->{{3.443451373078125*^9, 3.443451407625*^9}, {
   3.44345150084375*^9, 3.443451909640625*^9}, 3.445099074*^9, {
   3.44946647159375*^9, 3.4494664766875*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DTLocalValue", "[", "n", "]"}], "==", "9"}]], "Input",
 CellChangeTimes->{{3.44345191409375*^9, 3.44345192128125*^9}, 
   3.445099085375*^9}],

Cell["\<\
Because DTLocalValue returns the variable $Unknown if it would otherwise \
fail, a conditional such as the above, will only fire when a local variable \
'n' actually exists.\
\>", "Text",
 CellChangeTimes->{{3.443451937890625*^9, 3.443452065328125*^9}, {
   3.444159797453125*^9, 3.44415979778125*^9}, {3.44415999771875*^9, 
   3.444160003421875*^9}, 3.44509909409375*^9}],

Cell["\<\
Remember that you may need to include a context in the name you give to \
DTLocalValue. For example, if you are debugging a package, and 'n' is a \
private symbol, local to a module, then you might need to execute:\
\>", "Text",
 CellChangeTimes->{{3.443452079109375*^9, 3.4434521635625*^9}, {
  3.449466505140625*^9, 3.449466613125*^9}}],

Cell[BoxData[
 RowBox[{"DTLocalValue", "[", "myPackage`Private`n", "]"}]], "Input",
 CellChangeTimes->{{3.44946661853125*^9, 3.44946663534375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Code timing", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.44345135315625*^9, 3.4434513634375*^9}, {
  3.452362304171875*^9, 3.452362314921875*^9}, {3.505118073640625*^9, 
  3.50511807965625*^9}}],

Cell["\<\
When you run your code inside DebugTrace, it will run a little slower than \
normal. This is true more or less for all debuggers of all programming \
languages. Therefore DebugTrace is not an ideal environment in which to study \
code performance. However, often a program spends almost all its time in one \
small section of code \[Dash] possibly just one function call, and stepping \
through the code is a way to find such speed bumps! However, to actually \
record the time used since the previous DebugTrace pause, press the \
\"Execution Timing\" button so that it switches on to its red colour. The \
extra information is added to the results notebook.\
\>", "Text",
 CellChangeTimes->{{3.50511811075*^9, 3.505118366703125*^9}, {
  3.505118408484375*^9, 3.505118472640625*^9}, {3.50511850996875*^9, 
  3.505118532625*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other useful functions", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.44345135315625*^9, 3.4434513634375*^9}, {
  3.452362304171875*^9, 3.452362314921875*^9}}],

Cell[TextData[{
 "Once the DebugTrace package has been loaded, the function DTPrint can be \
used with or without the rest of the debugger, to print out intermediate \
values in a convenient way for debugging. See ",
 Cell[BoxData[
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTPrint\"\>", True->
     StyleBox["\<\"DTPrint\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTPrint",
    ButtonNote->"paclet:DebugTrace/ref/DTPrint"],
   Annotation[#, "paclet:DebugTrace/ref/DTPrint", "Hyperlink"]& ]],
  CellChangeTimes->{3.452362567375*^9, 3.452362657015625*^9}],
 " for details."
}], "Text",
 CellChangeTimes->{{3.4523623539375*^9, 3.452362480234375*^9}, {
  3.45236259778125*^9, 3.452362614265625*^9}, {3.45236267009375*^9, 
  3.452362676359375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dynamic execution", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.44345247775*^9, 3.4434524786875*^9}, {
  3.449466708296875*^9, 3.449466719125*^9}}],

Cell[TextData[{
 "Traditionally (i.e. prior to version 6!) the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " kernel processed your code when a cell in the frontend was executed. This \
is a slight simplification, but it was nearly true. Versions 6 and 7 contain \
dynamic elements that can invoke the execution of code in the kernel under a \
wide variety of situations \[Dash] such as when a user drags a slider, \
presses a button, or interacts with parts of a graphic object. Such programs \
are obviously more challenging to debug! DebugTrace has features that can \
help with this process, but it is much harder work."
}], "Text",
 CellChangeTimes->{{3.449466754375*^9, 3.449467077109375*^9}, {
  3.4494671125*^9, 3.44946712959375*^9}}],

Cell["\<\
\[FilledSmallCircle]  Many programs use dynamic code \[Dash] which may be as \
simple as a Manipulate \[Dash] to display the operation of functions that are \
quite conventional. For example, in the ToyDynamicExample.nb program, you \
have a quite conventional Mathematica function \[Dash] harm \[Dash] that \
assembles a few harmonics of a function, but it uses Dynamic code to exhibit \
the result. Obviously, this function is too simple to require debugging, but \
it is supposed to stand for something of greater complexity. If you suspected \
there is a fault in the function harm, it would be preferable to test and fix \
this function separately, using the facilities of DebugTrace, and then try \
the result in the dynamic code. The rest of this section discusses how to \
debug dynamic code where this option is not available.\
\>", "Text",
 CellChangeTimes->{{3.44946715109375*^9, 3.4494676080625*^9}, 
   3.449508980588935*^9}],

Cell["\<\
\[FilledSmallCircle]  Open the file ToyDynamicExample.nb, supplied with the \
package. A function 'harm' is defined to calculate the sum of a Sin wave and \
a few harmonics. This is followed by a Manipulate command that plots the \
result with sliders to adjust the harmonic content. The Manipulate command \
also contains a button to play the result as a sound.\
\>", "Text",
 CellChangeTimes->{{3.449496221265625*^9, 3.449496334578125*^9}, {
  3.449496378109375*^9, 3.44949652278125*^9}}],

Cell["\<\
\[FilledSmallCircle]  As with a non-dynamic debugging session, the first step \
is to click into the notebook (to ensure it is selected), and press the \
\"Debug this notebook\" button. Step through the notebook (or simply use \
\"Run until break\"). As you will see, the graphic will be generated \[Dash] \
which obviously means that the function 'harm' has been called many times \
\[Dash] but this is invisible to DebugTrace. Note that the dynamic graphic is \
output like any other, and will be placed in the results notebook.\
\>", "Text",
 CellChangeTimes->{{3.449496572984375*^9, 3.4494968288125*^9}, {
  3.44951005026081*^9, 3.449510078432685*^9}}],

Cell[TextData[{
 "\[FilledSmallCircle]  The ",
 StyleBox["crucial",
  FontVariations->{"Underline"->True}],
 " thing to realise about debugging dynamic code, is that it is impossible to \
pause such code. Dynamic code is supposed to execute in a timely fashion, and \
not to involve other interactions with the frontend. There are various \
options which can help to ease these restrictions, but in practice, it is \
best to accept that you cannot interact with your program while it is \
performing a dynamic task. When DebugTrace encounters a breakpoint in dynamic \
code, it does not pause, or even print out any information, it just records \
some information that can be retrieved later."
}], "Text",
 CellChangeTimes->{{3.449496852765625*^9, 3.449497051640625*^9}, {
  3.449497093984375*^9, 3.4494972335625*^9}, {3.44950899157331*^9, 
  3.449508999557685*^9}}],

Cell["\<\
\[FilledSmallCircle]  Having run the notebook through the debugger, it is now \
time to explore the dynamic behaviour of the code. Toggle the \"Record N \
dynamics\" on, and click on line 4 to select a breakpoint at that point. Now \
move a slider, so that the harm function is called (many times!).  The first \
N breakpoints will be recorded. The recording process is moderately \
expensive, so it is important not to collect excessive amounts of dynamic \
results. You choose the number of steps to record by setting N.\
\>", "Text",
 CellChangeTimes->{{3.449507659807685*^9, 3.44950780816706*^9}, {
  3.449507927401435*^9, 3.44950797863581*^9}, {3.449772490546875*^9, 
  3.449772494765625*^9}, {3.44977259096875*^9, 3.44977269515625*^9}}],

Cell["\<\
\[FilledSmallCircle]  It is now time to retrieve the breakpoint information \
using the DTDynamicResults function:\
\>", "Text",
 CellChangeTimes->{{3.44950798932331*^9, 3.44950801451081*^9}, {
  3.44950820319831*^9, 3.44950821507331*^9}, {3.44950913457331*^9, 
  3.44950914910456*^9}, {3.44977251109375*^9, 3.449772572640625*^9}}],

Cell[BoxData[
 RowBox[{"DTDynamicResults", "[", "]"}]], "Input",
 CellChangeTimes->{{3.449508081151435*^9, 3.44950808866706*^9}},
 CellLabel->"In[3]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "5", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 5, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "12", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 12, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "19", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 19, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "26", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 26, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "33", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 33, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "40", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 40, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:39.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "47", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:39.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 47, ")"],
    Editable->False], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"...", "..."}], "..."}], "..."}], "..."}], ".."}], 
    "\[IndentingNewLine]", "\[IndentingNewLine]", 
    InterpretationBox[
     RowBox[{
     "\"\<21:41:41.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
      "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
      "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", 
      "\"\< (Step \>\"", "\[InvisibleSpace]", "9049", "\[InvisibleSpace]", 
      "\"\<)\>\""}],
     SequenceForm[
     "21:41:41.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 9049, 
      ")"],
     Editable->False]}], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:41.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "9056", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:41.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 9056, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:41.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "9063", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:41.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 9063, ")"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
    "\"\<21:41:41.2\>\"", "\[InvisibleSpace]", "\"\< Line \>\"", 
     "\[InvisibleSpace]", "4", "\[InvisibleSpace]", 
     "\"\< (Passing breakpoint )\>\"", "\[InvisibleSpace]", "\"\< (Step \>\"",
      "\[InvisibleSpace]", "9070", "\[InvisibleSpace]", "\"\<)\>\""}],
    SequenceForm[
    "21:41:41.2", " Line ", 4, " (Passing breakpoint )", " (Step ", 9070, ")"],
    Editable->False]}], "}"}]], "Input",
 CellChangeTimes->{{3.449508176995185*^9, 3.449508190807685*^9}},
 CellLabel->"Out[3]="],

Cell["\<\
Note that the output list has been shortened for convenience. The information \
is essentially similar to what could have been obtained in an ordinary debug \
session, except, of course, that no interaction was possible. Because the \
speed of updating is often crucial to the appearance of a dynamic construct, \
the time of each breakpoint is recorded to the nearest tenth of a second.\
\>", "Text",
 CellChangeTimes->{{3.44950824507331*^9, 3.449508331901435*^9}, {
   3.44950837069831*^9, 3.449508528276435*^9}, 3.44950900679206*^9}],

Cell["\<\
\[FilledSmallCircle] It is also possible to obtain a print of this \
information more simply by using the \"Print Dynamics\" button. This button \
also clears the dynamic information.\
\>", "Text",
 CellChangeTimes->{{3.456829284734375*^9, 3.45682936515625*^9}}],

Cell["\<\
\[FilledSmallCircle]  Although it is not possible to pause at breakpoints in \
dynamic code, it is possible to store one or more arbitrary expressions to be \
evaluated at a breakpoint. If you hold down the shift key as you click on the \
breakpoint button, you will display a box that can be used to enter such \
expressions. For example, at the breakpoint at line 4, you might want to \
attach the expression DTLocalValue[ans].\
\>", "Text",
 CellChangeTimes->{{3.44950854454206*^9, 3.44950875460456*^9}, 
   3.449509012901435*^9}],

Cell["\<\
\[FilledSmallCircle] If the break on messages button is lit, messages are \
also recorded, together with the results that would have been obtained by \
pressing the \"Trace\" button.\
\>", "Text",
 CellChangeTimes->{{3.456829147484375*^9, 3.45682925278125*^9}}],

Cell["\<\
\[FilledSmallCircle]  Expressions can be attached to breakpoints in this way, \
whether or not you are working on dynamic code, but they are obviously most \
useful in dynamic code, where no hands-on interaction is possible.\
\>", "Text",
 CellChangeTimes->{{3.449508762057685*^9, 3.449508846245185*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tips", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.44345247775*^9, 3.4434524786875*^9}}],

Cell["\<\
\[FilledSmallCircle]  If your program uses random numbers, it will help to \
use SeedRandom at the start of your code, to ensure that each run of the \
debugger produces identical results, and follows identical paths within the \
program. The call to SeedRandom can always be removed if not required, once \
the program is working correctly.\
\>", "Text",
 CellChangeTimes->{{3.443452496328125*^9, 3.443452636734375*^9}, {
  3.44345316628125*^9, 3.443453180875*^9}}],

Cell[TextData[{
 "\[FilledSmallCircle]  Debuggers for other languages, often contain a list \
of watched variables (or expressions) that are displayed with their latest \
values at each step. With ",
 StyleBox["Mathematica, ",
  FontSlant->"Italic"],
 "it is, of course, trivial to do this for yourself. For example, if you \
wished to watch the array 'output' as you stepped the program, simply execute:"
}], "Text",
 CellChangeTimes->{{3.4434532615625*^9, 3.443453275515625*^9}, {
  3.443453923078125*^9, 3.443454019296875*^9}, {3.449509056151435*^9, 
  3.44950907751081*^9}}],

Cell[BoxData[
 RowBox[{"Dynamic", "[", "output", "]"}]], "Input",
 CellChangeTimes->{{3.443454101921875*^9, 3.443454105890625*^9}}],

Cell["\<\
The output cell from this will update as the program progresses.
However, if you wish to display a DTLocalValue function call, Dynamic will \
not be able to automatically track the changes as the program goes in and out \
of Modules, so it is necessary to give it a refresh time, for example:\
\>", "Text",
 CellChangeTimes->{{3.44345414190625*^9, 3.443454165390625*^9}, {
   3.4441594579375*^9, 3.444159595109375*^9}, 3.445099112765625*^9}],

Cell[BoxData[
 RowBox[{"Dynamic", "[", 
  RowBox[{
   RowBox[{"DTLocalValue", "[", "hope", "]"}], ",", 
   RowBox[{"UpdateInterval", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellChangeTimes->{3.44509912315625*^9}],

Cell["\<\
\[FilledSmallCircle]  Always try to debug the simplest version of your \
program operating on the smallest datasets. For example, it will be much \
easier to debug a program that operates on vectors of size 10, rather than if \
they are of size 1000000! It is often worth creating files of artificial data \
that are deliberately small in order to achieve this.\
\>", "Text",
 CellChangeTimes->{{3.44415848246875*^9, 3.444158608109375*^9}, {
  3.444158651796875*^9, 3.444158687421875*^9}, {3.4441596698125*^9, 
  3.44415967209375*^9}}],

Cell["\<\
\[FilledSmallCircle]  If you are able to work on a machine with two monitors, \
this will help considerably because you can view the source code notebook \
with line numbers on one screen, and the results notebook on the other.\
\>", "Text",
 CellChangeTimes->{{3.44610264346875*^9, 3.44610278040625*^9}, {
  3.446103528921875*^9, 3.44610358003125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Current limitations", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}}],

Cell["\<\
\[FilledSmallCircle]  Cannot handle programs that use the new parallel \
processing facilities, introduced at version 7.0. This limitation may be \
addressed later.\
\>", "Text",
 CellChangeTimes->{{3.44610599975*^9, 3.44610606246875*^9}}],

Cell["\<\
\[FilledSmallCircle] DebugTrace takes the code in each cell of your notebook, \
and recursively searches it for constructs that it can annotate. Depending on \
your coding style, this may be more or less effective. For example, it will \
find the body of a Module and break it into steps, however it currently \
assumes functional constructs, such as Map, are not worth processing more \
deeply. This may change in later versions.

In general, if DebugTrace does not know how to handle a construct, it will \
leave it alone. Typically, this may result in gray line numbers in the source \
window, and a region of code that executes as a single step.

DebugTrace works by modifying the code as it is sent to the kernel. You can \
view this modification by querying functions defined using the debugger:\
\>", "Text",
 CellChangeTimes->{{3.44303739921875*^9, 3.443037913796875*^9}, {
  3.44303804784375*^9, 3.443038048671875*^9}, {3.44303862859375*^9, 
  3.4430387293125*^9}, {3.445266361828125*^9, 3.445266363203125*^9}, {
  3.446049045296875*^9, 3.44604904778125*^9}, {3.446103711359375*^9, 
  3.446103721890625*^9}, {3.44610594746875*^9, 3.446105979078125*^9}}],

Cell[BoxData[
 RowBox[{"?", "CalculateLegendre"}]], "Input",
 CellChangeTimes->{{3.44303793103125*^9, 3.443037931625*^9}}],

Cell["\<\
In very unusual circumstances, a program might query itself internally \
(typically by calling DownValues), and fail under DebugTrace. More typically, \
however, DownValues is used to examine a new function created on the fly, and \
not present in the source file. This should work without problem with \
DebugTrace.\
\>", "Text",
 CellChangeTimes->{{3.443037955171875*^9, 3.4430380346875*^9}, {
   3.445266472*^9, 3.445266650921875*^9}, 3.446635921859375*^9}],

Cell["\<\
Some programs construct expressions, and then execute them. Such expressions \
will execute without benefit of the debugger, but otherwise, this should not \
cause any problems.\
\>", "Text",
 CellChangeTimes->{{3.443038149640625*^9, 3.443038215484375*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["See Also", "Section",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.4422355281752367`*^9, 3.4422355358362527`*^9}, {3.44265215084375*^9, 
  3.442652160140625*^9}, {3.442952775171875*^9, 3.44295278434375*^9}, {
  3.442954590421875*^9, 3.44295460484375*^9}, {3.443019616171875*^9, 
  3.4430196250625*^9}, {3.4430205540625*^9, 3.443020585765625*^9}, {
  3.44302121403125*^9, 3.443021224546875*^9}, {3.443037371140625*^9, 
  3.443037378796875*^9}, {3.452362739*^9, 3.452362744046875*^9}}],

Cell[BoxData[
 RowBox[{
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTLocalValue\"\>", True->
     StyleBox["\<\"DTLocalValue\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTLocalValue",
    ButtonNote->"paclet:DebugTrace/ref/DTLocalValue"],
   Annotation[#, "paclet:DebugTrace/ref/DTLocalValue", "Hyperlink"]& ], " ", 
  "\[CenterDot]", " ", 
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTDebugExecute\"\>", True->
     StyleBox["\<\"DTDebugExecute\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTDebugExecute",
    ButtonNote->"paclet:DebugTrace/ref/DTDebugExecute"],
   Annotation[#, "paclet:DebugTrace/ref/DTDebugExecute", "Hyperlink"]& ], " ",
   "\[CenterDot]", " ", 
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTRunDebugger\"\>", True->
     StyleBox["\<\"DTRunDebugger\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTRunDebugger",
    ButtonNote->"paclet:DebugTrace/ref/DTRunDebugger"],
   Annotation[#, "paclet:DebugTrace/ref/DTRunDebugger", "Hyperlink"]& ], " ", 
  "\[CenterDot]", " ", 
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTDynamicResults\"\>", True->
     StyleBox["\<\"DTDynamicResults\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTDynamicResults",
    ButtonNote->"paclet:DebugTrace/ref/DTDynamicResults"],
   Annotation[#, "paclet:DebugTrace/ref/DTDynamicResults", "Hyperlink"]& ], 
  " ", "\[CenterDot]", " ", 
  TagBox[
   ButtonBox[
    PaneSelectorBox[{False->"\<\"DTPrint\"\>", True->
     StyleBox["\<\"DTPrint\"\>", "HyperlinkActive"]}, Dynamic[
      CurrentValue["MouseOver"]],
     BaselinePosition->Baseline,
     FrameMargins->0,
     ImageSize->Automatic],
    BaseStyle->"Link",
    ButtonData->"paclet:DebugTrace/ref/DTPrint",
    ButtonNote->"paclet:DebugTrace/ref/DTPrint"],
   Annotation[#, "paclet:DebugTrace/ref/DTPrint", "Hyperlink"]& ]}]], "Text",
 CellChangeTimes->{{3.45236302334375*^9, 3.452363079140625*^9}, {
  3.452363116328125*^9, 3.45236315359375*^9}}]
}, Open  ]]
}, Open  ]]
},
Saveable->False,
Selectable->True,
WindowSize->{717, 741},
WindowMargins->{{196, Automatic}, {Automatic, 0}},
WindowFrame->"Normal",
Visible->True,
DockedCells->FEPrivate`FrontEndResource["FEExpressions", "HelpViewerToolbar"],
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, 
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"]}, {
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"], Inherited, 
   Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}, {
  Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowCellBracket->True,
ShowSelection->True,
TaggingRules->{"SearchTextTranslated" -> "", "SearchText" -> ""},
ShowCellLabel->False,
Magnification->1.,
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 144, 2, 85, "Chapter"],
Cell[CellGroupData[{
Cell[736, 28, 152, 2, 63, "Section"],
Cell[891, 32, 4068, 63, 519, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4996, 100, 255, 3, 63, "Section"],
Cell[5254, 105, 1162, 24, 103, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6453, 134, 203, 3, 70, "Section"],
Cell[6659, 139, 428, 7, 70, "Text"],
Cell[7090, 148, 1035, 23, 70, "Text"],
Cell[8128, 173, 310, 4, 70, "Text"],
Cell[8441, 179, 494, 9, 70, "Text"],
Cell[8938, 190, 859, 14, 70, "Text"],
Cell[9800, 206, 1012, 15, 70, "Text"],
Cell[10815, 223, 640, 14, 70, "Print"],
Cell[11458, 239, 573, 9, 70, "Text"],
Cell[12034, 250, 640, 14, 70, "Print"],
Cell[12677, 266, 637, 14, 70, "Print"],
Cell[13317, 282, 644, 14, 70, "Print"],
Cell[13964, 298, 726, 16, 70, "Print"],
Cell[14693, 316, 741, 17, 70, "Print"],
Cell[15437, 335, 643, 14, 70, "Print"],
Cell[16083, 351, 643, 14, 70, "Print"],
Cell[16729, 367, 695, 15, 70, "Print"],
Cell[17427, 384, 639, 14, 70, "Print"],
Cell[18069, 400, 644, 14, 70, "Print"],
Cell[18716, 416, 696, 15, 70, "Print"],
Cell[19415, 433, 645, 14, 70, "Print"],
Cell[20063, 449, 645, 14, 70, "Print"],
Cell[20711, 465, 698, 15, 70, "Print"],
Cell[21412, 482, 645, 14, 70, "Print"],
Cell[22060, 498, 644, 14, 70, "Print"],
Cell[22707, 514, 696, 15, 70, "Print"],
Cell[23406, 531, 644, 14, 70, "Print"],
Cell[24053, 547, 641, 14, 70, "Print"],
Cell[24697, 563, 697, 15, 70, "Print"],
Cell[25397, 580, 643, 14, 70, "Print"],
Cell[26043, 596, 644, 14, 70, "Print"],
Cell[26690, 612, 698, 15, 70, "Print"],
Cell[27391, 629, 645, 14, 70, "Print"],
Cell[28039, 645, 645, 14, 70, "Print"],
Cell[28687, 661, 698, 15, 70, "Print"],
Cell[29388, 678, 645, 14, 70, "Print"],
Cell[30036, 694, 645, 14, 70, "Print"],
Cell[30684, 710, 698, 15, 70, "Print"],
Cell[31385, 727, 645, 14, 70, "Print"],
Cell[32033, 743, 645, 14, 70, "Print"],
Cell[32681, 759, 696, 15, 70, "Print"],
Cell[33380, 776, 644, 14, 70, "Print"],
Cell[34027, 792, 641, 14, 70, "Print"],
Cell[34671, 808, 655, 11, 70, "Message"],
Cell[35329, 821, 728, 16, 70, "Print"],
Cell[36060, 839, 647, 14, 70, "Print"],
Cell[36710, 855, 2540, 88, 70, "Print"],
Cell[39253, 945, 642, 14, 70, "Print"],
Cell[39898, 961, 380, 10, 70, "Print"],
Cell[40281, 973, 408, 8, 70, "Print"],
Cell[40692, 983, 247, 5, 70, "Text"],
Cell[40942, 990, 727, 12, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41706, 1007, 346, 5, 70, "Section"],
Cell[42055, 1014, 797, 13, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42889, 1032, 315, 4, 70, "Section"],
Cell[43207, 1038, 375, 7, 70, "Text"],
Cell[43585, 1047, 623, 14, 70, "Print"],
Cell[44211, 1063, 633, 11, 70, "Message"],
Cell[44847, 1076, 674, 15, 70, "Print"],
Cell[45524, 1093, 505, 15, 70, "Print"],
Cell[46032, 1110, 669, 18, 70, "Print"],
Cell[46704, 1130, 374, 8, 70, "Print"],
Cell[47081, 1140, 96, 2, 70, "Output"],
Cell[47180, 1144, 764, 12, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47981, 1161, 425, 6, 70, "Section"],
Cell[48409, 1169, 915, 14, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49361, 1188, 371, 5, 70, "Section"],
Cell[49735, 1195, 1037, 15, 70, "Text"],
Cell[50775, 1212, 172, 4, 70, "Text"],
Cell[50950, 1218, 601, 16, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51588, 1239, 429, 6, 70, "Section"],
Cell[52020, 1247, 439, 8, 70, "Text"],
Cell[52462, 1257, 177, 3, 70, "Input"],
Cell[52642, 1262, 162, 3, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52841, 1270, 483, 6, 70, "Section"],
Cell[53327, 1278, 282, 5, 70, "Text"],
Cell[53612, 1285, 142, 2, 70, "Input"],
Cell[53757, 1289, 211, 4, 70, "Input"],
Cell[53971, 1295, 224, 4, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54232, 1304, 395, 5, 70, "Section"],
Cell[54630, 1311, 498, 8, 70, "Text"],
Cell[55131, 1321, 458, 7, 70, "Text"],
Cell[55592, 1330, 197, 4, 70, "Input"],
Cell[55792, 1336, 479, 8, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56308, 1349, 443, 6, 70, "Section"],
Cell[56754, 1357, 911, 16, 70, "Text"],
Cell[CellGroupData[{
Cell[57690, 1377, 372, 10, 70, "Input"],
Cell[58065, 1389, 174, 6, 70, "Print"]
}, Open  ]],
Cell[58254, 1398, 439, 7, 70, "Text"],
Cell[58696, 1407, 500, 8, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59233, 1420, 589, 8, 70, "Section"],
Cell[59825, 1430, 1130, 18, 70, "Text"],
Cell[60958, 1450, 612, 11, 70, "Text"],
Cell[61573, 1463, 695, 11, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62305, 1479, 535, 7, 70, "Section"],
Cell[62843, 1488, 902, 14, 70, "Text"],
Cell[63748, 1504, 176, 4, 70, "Input"],
Cell[63927, 1510, 382, 7, 70, "Text"],
Cell[64312, 1519, 348, 6, 70, "Text"],
Cell[64663, 1527, 147, 2, 70, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64847, 1534, 620, 9, 70, "Section"],
Cell[65470, 1545, 839, 13, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[66346, 1563, 583, 8, 70, "Section"],
Cell[66932, 1573, 931, 22, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67900, 1600, 572, 8, 70, "Section"],
Cell[68475, 1610, 750, 13, 70, "Text"],
Cell[69228, 1625, 948, 14, 70, "Text"],
Cell[70179, 1641, 500, 8, 70, "Text"],
Cell[70682, 1651, 666, 10, 70, "Text"],
Cell[71351, 1663, 866, 15, 70, "Text"],
Cell[72220, 1680, 752, 11, 70, "Text"],
Cell[72975, 1693, 341, 6, 70, "Text"],
Cell[73319, 1701, 152, 3, 70, "Input"],
Cell[73474, 1706, 5101, 112, 70, "Input"],
Cell[78578, 1820, 546, 8, 70, "Text"],
Cell[79127, 1830, 272, 5, 70, "Text"],
Cell[79402, 1837, 543, 9, 70, "Text"],
Cell[79948, 1848, 271, 5, 70, "Text"],
Cell[80222, 1855, 314, 5, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[80573, 1865, 513, 7, 70, "Section"],
Cell[81089, 1874, 476, 8, 70, "Text"],
Cell[81568, 1884, 578, 11, 70, "Text"],
Cell[82149, 1897, 131, 2, 70, "Input"],
Cell[82283, 1901, 451, 7, 70, "Text"],
Cell[82737, 1910, 210, 5, 70, "Input"],
Cell[82950, 1917, 545, 9, 70, "Text"],
Cell[83498, 1928, 363, 6, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83898, 1939, 488, 7, 70, "Section"],
Cell[84389, 1948, 249, 5, 70, "Text"],
Cell[84641, 1955, 1172, 19, 70, "Text"],
Cell[85816, 1976, 122, 2, 70, "Input"],
Cell[85941, 1980, 470, 8, 70, "Text"],
Cell[86414, 1990, 267, 5, 70, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86718, 2000, 517, 7, 70, "Section"],
Cell[87238, 2009, 2637, 67, 70, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

