(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      5505,        180]
NotebookOptionsPosition[      3244,        107]
NotebookOutlinePosition[      5065,        163]
CellTagsIndexPosition[      4991,        158]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "DTLocalValue ",
 StyleBox["(part of the DebugTrace package)",
  FontSize->12]
}], "Chapter",
 CellChangeTimes->{{3.4422352966623373`*^9, 3.442235300928472*^9}, {
  3.44731239346875*^9, 3.447312399453125*^9}, {3.447390701375*^9, 
  3.447390712625*^9}, {3.447390810796875*^9, 3.4473908173125*^9}},
 CellTags->"DTLocalValue"],

Cell[TextData[{
 "The DTLocalValue function searches for the value of a local variable or \
pattern argument to a function. It is meant to be used while the debugger is \
paused, or in conditional break expressions. ",
 StyleBox["Never",
  FontVariations->{"Underline"->True}],
 " use this function inside normal code."
}], "Text",
 CellChangeTimes->{{3.447390957390625*^9, 3.447391254703125*^9}}],

Cell["\<\
For example, suppose the debugger is paused on the assignment to myvar in the \
following code:\
\>", "Text",
 CellChangeTimes->{{3.44739130946875*^9, 3.447391385984375*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"myFunc", "[", "x_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "tmp", "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"tmp", "=", "42"}], ";", "\[IndentingNewLine]", 
       RowBox[{"myvar", "=", "tmp"}], ";"}]}], "\[IndentingNewLine]", "]"}]}],
    ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"myFunc", "[", "30", "]"}]}], "Input",
 CellChangeTimes->{{3.447391392265625*^9, 3.447391454078125*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DTLocalValue", "[", "x", "]"}]], "Input",
 CellChangeTimes->{{3.4473914695625*^9, 3.447391513890625*^9}},
 CellLabel->"In[2]:="],

Cell[BoxData["30"], "Output",
 CellChangeTimes->{3.44739149834375*^9},
 CellLabel->"Out[2]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DTLocalValue", "[", "tmp", "]"}]], "Input",
 CellChangeTimes->{{3.4473914695625*^9, 3.447391562125*^9}},
 CellLabel->"In[3]:="],

Cell[BoxData["42"], "Output",
 CellChangeTimes->{3.44739149834375*^9, 3.447391546125*^9},
 CellLabel->"Out[3]="]
}, Open  ]],

Cell["\<\
DTLocalValue always returns the value of the innermost variable with the \
required name, if there are several such names in use,  and it returns a \
Unique variable name if no suitable variable exists. \
\>", "Text",
 CellChangeTimes->{{3.447391583890625*^9, 3.447391772609375*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DTLocalValue", "[", "notThere", "]"}]], "Input",
 CellChangeTimes->{{3.44739170046875*^9, 3.44739170609375*^9}},
 CellLabel->"In[4]:="],

Cell[BoxData["DebugTrace`Private`$Unknown"], "Output",
 CellChangeTimes->{3.44739170821875*^9},
 CellLabel->"Out[4]="]
}, Open  ]]
}, Open  ]]
},
Saveable->False,
Selectable->True,
WindowSize->{717, 741},
WindowMargins->Automatic,
WindowFrame->"Normal",
Visible->True,
DockedCells->FEPrivate`FrontEndResource["FEExpressions", "HelpViewerToolbar"],
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, 
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"]}, {
   Cell[
    TextData[{"The Super Widget Package"}], "PageHeader"], Inherited, 
   Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}, {
  Inherited, 
   Cell[
    TextData[{"\[Copyright]2006 David Bailey"}], "PageFooter"], Inherited}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowCellBracket->True,
ShowSelection->True,
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> {2012, 12, 27, 18, 5, 18.296875`9.01495186115289}, "context" -> 
    "DebugTrace`", "keywords" -> {}, "index" -> True, "label" -> 
    "DebugTrace Package Paclet Symbol", "language" -> "en", "paclet" -> 
    "DebugTrace Package", "status" -> "", "summary" -> 
    "DTLocalValue[n] \[Dash] Returns the value of the most recently created \
local variable of the form n$<number>", "synonyms" -> {}, "title" -> 
    "DTLocalValue", "type" -> "Symbol", "uri" -> 
    "DebugTrace/ref/DTLocalValue"}, "SearchTextTranslated" -> ""},
ShowCellLabel->False,
Magnification->1.,
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "DTLocalValue"->{
  Cell[567, 22, 340, 8, 85, "Chapter",
   CellTags->"DTLocalValue"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"DTLocalValue", 4898, 152}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 340, 8, 85, "Chapter",
 CellTags->"DTLocalValue"],
Cell[910, 32, 397, 8, 39, "Text"],
Cell[1310, 42, 184, 4, 23, "Text"],
Cell[1497, 48, 530, 13, 128, "Input"],
Cell[CellGroupData[{
Cell[2052, 65, 152, 3, 27, "Input"],
Cell[2207, 70, 93, 2, 25, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2337, 77, 151, 3, 27, "Input"],
Cell[2491, 82, 112, 2, 25, "Output"]
}, Open  ]],
Cell[2618, 87, 293, 5, 39, "Text"],
Cell[CellGroupData[{
Cell[2936, 96, 159, 3, 27, "Input"],
Cell[3098, 101, 118, 2, 25, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

