(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     18355,        508]
NotebookOptionsPosition[     17857,        487]
NotebookOutlinePosition[     18300,        505]
CellTagsIndexPosition[     18257,        502]
WindowTitle->DebugTrace control panel
WindowFrame->Palette*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"DebugTrace", " "}],
   FontSize->18], 
  StyleBox[
   RowBox[{"Ver", " ", "1.50"}],
   FontSize->10]}]], "Text",
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {0, 0}},
 CellBracketOptions->{"Color"->RGBColor[0.269993, 0.308507, 0.6]},
 CellHorizontalScrolling->True,
 PageBreakAbove->True,
 PageBreakWithin->False,
 ShowAutoStyles->True,
 LineSpacing->{1.25, 0},
 AutoItalicWords->{},
 ScriptMinSize->9,
 ShowStringCharacters->False,
 FontFamily:>CurrentValue["PanelFontFamily"],
 FontSize:>CurrentValue["PanelFontSize"],
 Background->RGBColor[0, 1, 1]],

Cell[BoxData[
 TagBox[GridBox[{
    {
     TagBox[
      TooltipBox[
       ButtonBox[
        StyleBox["\<\"DebugTrace Help\"\>",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]],
        Appearance->Automatic,
        ButtonFunction:>(Needs["DebugTrace`"]; DebugTrace`DebugTraceHelp[]),
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Open help information\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Open help information", "Tooltip"]& ], "\[SpanFromLeft]"},
    {
     TagBox[
      TooltipBox[
       ButtonBox[
        StyleBox["\<\"Debug this notebook\"\>",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]],
        Appearance->Automatic,
        ButtonFunction:>(Needs["DebugTrace`"]; DebugTrace`DTRunDebugger[
           Evaluate[
            SelectedNotebook[]]]),
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Debug selected notebook\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Debug selected notebook", "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox[
        StyleBox["\<\"Debug selection\"\>",
         StripOnInput->False,
         LineColor->RGBColor[1, 0, 0],
         FrontFaceColor->RGBColor[1, 0, 0],
         BackFaceColor->RGBColor[1, 0, 0],
         GraphicsColor->RGBColor[1, 0, 0],
         FontColor->RGBColor[1, 0, 0]],
        Appearance->Automatic,
        ButtonFunction:>(Needs["DebugTrace`"]; 
         DebugTrace`Private`debug\[Breve]selection[]),
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Debug selected cell(s)\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Debug selected cell(s)", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Debug package\"\>",
        Appearance->Automatic,
        ButtonFunction:>(Needs["DebugTrace`"]; DebugTrace`DTRunDebugger[
           Evaluate[
            SelectedNotebook[]], "Package" -> True]),
        Enabled->True,
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Debug package or other called .m code\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Debug package or other called .m code", "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Kill kernel\"\>",
        Appearance->Automatic,
        ButtonFunction:>(If[
           ChoiceDialog["Also remove DebugTrace generated notebooks?"], 
           DebugTrace`Private`Close\[Breve]Generated\[Breve]Notebooks[]]; 
         Quit[]),
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Kill the kernel for a clean restart,\\noptionally removing all \
notebooks generated by DebugTrace\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, 
       "Kill the kernel for a clean restart,\noptionally removing all \
notebooks generated by DebugTrace", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Run until break\"\>",
        Appearance->Automatic,
        ButtonFunction:>If[
         DebugTrace`Private`in\[Breve]dialog, 
          DebugTrace`Private`in\[Breve]dialog = False; 
          DebugTrace`Private`stepping = False; ExitDialog[]],
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Continue program to completion, or until breakpoint\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Continue program to completion, or until breakpoint", 
       "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Step once\"\>",
        Appearance->Automatic,
        ButtonFunction:>If[
         DebugTrace`Private`in\[Breve]dialog, 
          DebugTrace`Private`in\[Breve]dialog = False; ExitDialog[]],
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run program to next step\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Run program to next step", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox[
        DynamicBox[ToBoxes[
          Style["Message interception", 
           If[
           DebugTrace`Private`message\[Breve]interception\[Breve]required, 
            Red, Black, Black]], StandardForm],
         ImageSizeCache->{104., {2., 8.}}],
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`message\[Breve]interception\[Breve]required = 
          DebugTrace`Private`message\[Breve]interception\[Breve]required =!= 
           True; DebugTrace`Private`message\[Breve]interception = 
          DebugTrace`Private`message\[Breve]interception\[Breve]required),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, Evaluator -> Automatic],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Toggle breakpoints on messages\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Toggle breakpoints on messages", "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Run to step\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`step\[Breve]at = Input["Enter step number:", 0]; 
         DebugTrace`Private`stepping = False; ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run to specific step number\\n(typically determined on a previous \
debug run)\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, 
       "Run to specific step number\n(typically determined on a previous \
debug run)", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Print N steps\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`step\[Breve]at = 
          DebugTrace`Private`multi\[Breve]step + 
           DebugTrace`Private`step\[Breve]counter; 
         DebugTrace`Private`stepping = False; 
         DebugTrace`Private`print\[Breve]steps\[Breve]regardless = True; 
         ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Print the next N steps but don't pause for interaction\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Print the next N steps but don't pause for interaction", 
       "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Run to function\"\>",
        Appearance->Automatic,
        ButtonFunction:>If[
          DebugTrace`Private`setup\[Breve]halt\[Breve]function[], 
          DebugTrace`Private`stepping = False; ExitDialog[]],
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run to next call to named function\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Run to next call to named function", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Run to variable\"\>",
        Appearance->Automatic,
        ButtonFunction:>If[
          DebugTrace`Private`setup\[Breve]halt\[Breve]variable[], 
          DebugTrace`Private`stepping = False; ExitDialog[]],
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run to next use or change of named variable\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Run to next use or change of named variable", 
       "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Skip N steps\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`step\[Breve]at = 
          DebugTrace`Private`multi\[Breve]step + 
           DebugTrace`Private`step\[Breve]counter; 
         DebugTrace`Private`stepping = False; ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run for N steps (default 100 - see Set N button)\\n without \
interaction\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, 
       "Run for N steps (default 100 - see Set N button)\n without \
interaction", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Interrupt\"\>",
        Appearance->Automatic,
        ButtonFunction:>(DebugTrace`Private`stepping = True),
        Enabled->Dynamic[
          Not[DebugTrace`Private`in\[Breve]dialog], SynchronousUpdating -> 
          False, TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Preemptive"],
       "\"Interupt a running program\\n(e.g. after pressing Continue)\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, 
       "Interupt a running program\n(e.g. after pressing Continue)", 
       "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Quit\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`task = DebugTrace`Private`task\[Breve]abort; 
         ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Stop the program completely\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Stop the program completely", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Trace\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`task = DebugTrace`Private`task\[Breve]trace; 
         ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Print current stack of function call information\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Print current stack of function call information", 
       "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Set N\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`multi\[Breve]step = 
          Input["Enter N for block steps:", 
            DebugTrace`Private`multi\[Breve]step]; Null),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Adjust N, the jump size\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Adjust N, the jump size", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox[
        DynamicBox[ToBoxes[
          Style["Record N dynamics", 
           If[
           DebugTrace`Private`dynamic\[Breve]debug\[Breve]count > 0, Red, 
            Black, Black]], StandardForm],
         ImageSizeCache->{94., {2., 9.}}],
        Appearance->Automatic,
        ButtonFunction:>DebugTrace`Private`debug\[Breve]dynamics[],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Toggle continuous debugging for dynamics\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Toggle continuous debugging for dynamics", "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Step over\"\>",
        Appearance->Automatic,
        ButtonFunction:>If[
         DebugTrace`Private`in\[Breve]dialog, 
          DebugTrace`Private`in\[Breve]dialog = False; 
          DebugTrace`Private`task = 
           DebugTrace`Private`task\[Breve]step\[Breve]over; ExitDialog[]],
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Run program to next step\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Run program to next step", "Tooltip"]& ]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Print dynamics\"\>",
        Appearance->Automatic,
        ButtonFunction:>DebugTrace`Private`print\[Breve]dynamics[],
        Enabled->Dynamic[DebugTrace`Private`dynamic\[Breve]results =!= {}],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Toggle continuous debugging for dynamics\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Toggle continuous debugging for dynamics", "Tooltip"]& ], 
     ButtonBox[
      DynamicBox[ToBoxes[
        Style[
         If[
         DebugTrace`Private`SWP\[Breve]Modal === True, "SWP break", 
          "Private"], 
         If[DebugTrace`Private`SWP\[Breve]Modal === True, Red, Gray]], 
        StandardForm],
       ImageSizeCache->{35., {0., 8.}}],
      Appearance->Automatic,
      ButtonFunction:>If[
       DebugTrace`Private`SWP\[Breve]Modal, 
        DebugTrace`Private`stepping = True; 
        ToExpression[
         "SuperWidgetPackage`Private`debugger\[Breve]interrupt=True"]],
      Enabled->Dynamic[
        Not[DebugTrace`Private`in\[Breve]dialog]],
      Evaluator->Automatic,
      Method->"Preemptive"]},
    {
     TagBox[
      TooltipBox[
       ButtonBox["\<\"Conditionals\"\>",
        Appearance->Automatic,
        ButtonFunction:>(
         DebugTrace`Private`task = 
          DebugTrace`Private`task\[Breve]conditionals; ExitDialog[]),
        Enabled->Dynamic[
         DebugTrace`Private`in\[Breve]dialog, SynchronousUpdating -> False, 
          TrackedSymbols -> Full],
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Add/remove conditional breaakpoints\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Add/remove conditional breaakpoints", "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       ButtonBox[
        DynamicBox[ToBoxes[
          Style["Execution timing", 
           If[DebugTrace`Private`step\[Breve]timing, Red, Black, Black]], 
          StandardForm],
         ImageSizeCache->{79., {2., 8.}}],
        Appearance->Automatic,
        ButtonFunction:>(DebugTrace`Private`step\[Breve]timing = 
         DebugTrace`Private`step\[Breve]timing =!= True),
        Evaluator->Automatic,
        Method->"Queued"],
       "\"Toggle reporting of code timing information\"",
       TooltipStyle->"TextStyling"],
      Annotation[#, "Toggle reporting of code timing information", 
       "Tooltip"]& ]},
    {
     ItemBox[
      StyleBox[
       DynamicBox[ToBoxes[If[
           Not[
            ValueQ[DebugTrace`Private`status\[Breve]info]], 
           DebugTrace`Private`status\[Breve]info = "INACTIVE"]; 
         StringJoin[" Status: ", DebugTrace`Private`status\[Breve]info], 
         StandardForm],
        ImageSizeCache->{90., {1., 8.}},
        TrackedSymbols:>Full],
       StripOnInput->False,
       LineColor->RGBColor[1, 0, 0],
       FrontFaceColor->RGBColor[1, 0, 0],
       BackFaceColor->RGBColor[1, 0, 0],
       GraphicsColor->RGBColor[1, 0, 0],
       FontColor->RGBColor[1, 0, 0]],
      Alignment->Left,
      StripOnInput->False], 
     ItemBox[
      DynamicBox[ToBoxes[If[
          Not[
           ValueQ[DebugTrace`Private`stepping\[Breve]info]], 
          DebugTrace`Private`stepping\[Breve]info = ""]; 
        DebugTrace`Private`stepping\[Breve]info, StandardForm],
       ImageSizeCache->{0., {0., 6.}},
       TrackedSymbols:>Full],
      Alignment->Left,
      StripOnInput->False]}
   },
   AutoDelete->False,
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]],
 CellMargins->{{0, 0}, {0, 0}},
 CellBracketOptions->{"Color"->RGBColor[0.269993, 0.308507, 0.6]},
 CellHorizontalScrolling->True,
 PageBreakAbove->True,
 PageBreakWithin->False,
 ShowAutoStyles->True,
 LineSpacing->{1.25, 0},
 AutoItalicWords->{},
 ScriptMinSize->9,
 ShowStringCharacters->False,
 FontFamily:>CurrentValue["PanelFontFamily"],
 FontSize:>CurrentValue["PanelFontSize"]]
},
WindowSize->All,
WindowMargins->{{Automatic, 32}, {Automatic, 18}},
WindowElements->{"MenuBar"},
WindowFrameElements->{"CloseBox"},
WindowTitle->"DebugTrace control panel",
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
StyleDefinitions->"Palette.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[596, 21, 603, 21, 36, InheritFromParent,
 PageBreakAbove->True,
 PageBreakWithin->False],
Cell[1202, 44, 16651, 441, 337, InheritFromParent,
 PageBreakAbove->True,
 PageBreakWithin->False]
}
]
*)

(* End of internal cache information *)
